"""
Python module generated from Java source file com.google.gson.FieldAttributes

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import *
from java.lang.reflect import Field
from java.lang.reflect import Type
from java.util import Arrays
from java.util import Objects
from typing import Any, Callable, Iterable, Tuple


class FieldAttributes:
    """
    A data object that stores attributes of a field.
    
    This class is immutable; therefore, it can be safely shared across threads.

    Author(s)
    - Joel Leitch

    Since
    - 1.4
    """

    def __init__(self, f: "Field"):
        """
        Constructs a Field Attributes object from the `f`.

        Arguments
        - f: the field to pull attributes from
        """
        ...


    def getDeclaringClass(self) -> type[Any]:
        """
        Gets the declaring Class that contains this field

        Returns
        - the declaring class that contains this field
        """
        ...


    def getName(self) -> str:
        """
        Gets the name of the field

        Returns
        - the name of the field
        """
        ...


    def getDeclaredType(self) -> "Type":
        """
        Returns the declared generic type of the field.
        
        For example, assume the following class definition:
        
        <pre class="code">
        public class Foo {
          private String bar;
          private List&lt;String&gt; red;
        }
        
        Type listParameterizedType = new TypeToken&lt;List&lt;String&gt;&gt;() {}.getType();
        ```
        
        This method would return `String.class` for the `bar` field and `listParameterizedType` for the `red` field.

        Returns
        - the specific type declared for this field
        """
        ...


    def getDeclaredClass(self) -> type[Any]:
        """
        Returns the `Class` object that was declared for this field.
        
        For example, assume the following class definition:
        
        <pre class="code">
        public class Foo {
          private String bar;
          private List&lt;String&gt; red;
        }
        ```
        
        This method would return `String.class` for the `bar` field and `List.class` for the `red` field.

        Returns
        - the specific class object that was declared for the field
        """
        ...


    def getAnnotation(self, annotation: type["T"]) -> "T":
        """
        Returns the `T` annotation object from this field if it exists; otherwise returns `null`.

        Arguments
        - annotation: the class of the annotation that will be retrieved

        Returns
        - the annotation instance if it is bound to the field; otherwise `null`
        """
        ...


    def getAnnotations(self) -> Iterable["Annotation"]:
        """
        Returns the annotations that are present on this field.

        Returns
        - an array of all the annotations set on the field

        Since
        - 1.4
        """
        ...


    def hasModifier(self, modifier: int) -> bool:
        """
        Returns `True` if the field is defined with the `modifier`.
        
        This method is meant to be called as:
        
        <pre class="code">
        boolean hasPublicModifier = fieldAttribute.hasModifier(java.lang.reflect.Modifier.PUBLIC);
        ```

        See
        - java.lang.reflect.Modifier
        """
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file java.io.UncheckedIOException

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from java.util import Objects
from typing import Any, Callable, Iterable, Tuple


class UncheckedIOException(RuntimeException):
    """
    Wraps an IOException with an unchecked exception.

    Since
    - 1.8
    """

    def __init__(self, message: str, cause: "IOException"):
        """
        Constructs an instance of this class.

        Arguments
        - message: the detail message, can be null
        - cause: the `IOException`

        Raises
        - NullPointerException: if the cause is `null`
        """
        ...


    def __init__(self, cause: "IOException"):
        """
        Constructs an instance of this class.

        Arguments
        - cause: the `IOException`

        Raises
        - NullPointerException: if the cause is `null`
        """
        ...


    def getCause(self) -> "IOException":
        """
        Returns the cause of this exception.

        Returns
        - the `IOException` which is the cause of this exception.
        """
        ...



"""
Python module generated from Java source file com.google.gson.internal.bind.TypeAdapterRuntimeTypeWrapper

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import Gson
from com.google.gson import TypeAdapter
from com.google.gson.internal.bind import *
from com.google.gson.reflect import TypeToken
from com.google.gson.stream import JsonReader
from com.google.gson.stream import JsonWriter
from java.io import IOException
from java.lang.reflect import Type
from java.lang.reflect import TypeVariable
from typing import Any, Callable, Iterable, Tuple


class TypeAdapterRuntimeTypeWrapper(TypeAdapter):

    def read(self, in: "JsonReader") -> "T":
        ...


    def write(self, out: "JsonWriter", value: "T") -> None:
        ...



"""
Python module generated from Java source file org.bukkit.entity.EvokerFangs

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class EvokerFangs(Entity):
    """
    Represents Evoker Fangs.
    """

    def getOwner(self) -> "LivingEntity":
        """
        Gets the LivingEntity which summoned the fangs.

        Returns
        - the LivingEntity which summoned the fangs
        """
        ...


    def setOwner(self, owner: "LivingEntity") -> None:
        """
        Sets the LivingEntity which summoned the fangs.

        Arguments
        - owner: the LivingEntity which summoned the fangs
        """
        ...


    def getAttackDelay(self) -> int:
        """
        Get the delay in ticks until the fang attacks.

        Returns
        - the delay
        """
        ...


    def setAttackDelay(self, delay: int) -> None:
        """
        Set the delay in ticks until the fang attacks.

        Arguments
        - delay: the delay, must be positive
        """
        ...



"""
Python module generated from Java source file org.bukkit.boss.DragonBattle

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit import Location
from org.bukkit.boss import *
from org.bukkit.entity import EnderCrystal
from org.bukkit.entity import EnderDragon
from typing import Any, Callable, Iterable, Tuple


class DragonBattle:
    """
    Represents a dragon battle state for a world with an end environment.
    """

    def getEnderDragon(self) -> "EnderDragon":
        """
        Get the EnderDragon active in this battle.
        
        Will return null if the dragon has been slain.

        Returns
        - the ender dragon. null if dead
        """
        ...


    def getBossBar(self) -> "BossBar":
        """
        Get the boss bar to be displayed for this dragon battle.

        Returns
        - the boss bar
        """
        ...


    def getEndPortalLocation(self) -> "Location":
        """
        Get the location of the end portal.
        
        This location will be at the center of the base (bottom) of the portal.

        Returns
        - the end portal location or null if not generated
        """
        ...


    def generateEndPortal(self, withPortals: bool) -> bool:
        """
        Generate the end portal.

        Arguments
        - withPortals: whether end portal blocks should be generated

        Returns
        - True if generated, False if already present
        """
        ...


    def hasBeenPreviouslyKilled(self) -> bool:
        """
        Check whether the first dragon has been killed already.

        Returns
        - True if killed before, False otherwise
        """
        ...


    def setPreviouslyKilled(self, previouslyKilled: bool) -> None:
        """
        Sets whether the first dragon has been killed already.
        
        If the dragon has not previously been killed, a portal will be generated
        when it is finally killed.

        Arguments
        - previouslyKilled: True if the dragon has been killed before, False
        otherwise
        """
        ...


    def initiateRespawn(self) -> None:
        """
        Try to initiate a respawn sequence to summon the dragon as though a player has
        placed 4 end crystals on the portal.
        """
        ...


    def initiateRespawn(self, enderCrystals: Iterable["EnderCrystal"]) -> bool:
        """
        Try to initiate a respawn sequence to summon the dragon.

        Arguments
        - enderCrystals: the EnderCrystal EnderCrystals to use in the
        respawn, or a null or empty list to render the respawn sequence
        uncancellable. null entries or crystals that do not reside in the same
        world as this dragon battle will be ignored.

        Returns
        - True if the respawn was initiated, False otherwise.
        """
        ...


    def getRespawnPhase(self) -> "RespawnPhase":
        """
        Get this battle's current respawn phase.

        Returns
        - the current respawn phase.
        """
        ...


    def setRespawnPhase(self, phase: "RespawnPhase") -> bool:
        """
        Set the dragon's respawn phase.
        
        This method will is unsuccessful if a dragon respawn is not in progress.

        Arguments
        - phase: the phase to set

        Returns
        - True if successful, False otherwise

        See
        - .initiateRespawn()
        """
        ...


    def resetCrystals(self) -> None:
        """
        Reset the crystals located on the obsidian pillars (remove their beam
        targets and invulnerability).
        """
        ...


    class RespawnPhase(Enum):
        """
        Represents a phase in the dragon respawn process.
        """

        START = 0
        """
        The crystal beams are directed upwards into the sky.
        """
        PREPARING_TO_SUMMON_PILLARS = 1
        """
        The crystal beams remain directed upwards.
        """
        SUMMONING_PILLARS = 2
        """
        The crystal beams are directed from pillar to pillar, regenerating
        their crystals if necessary.
        """
        SUMMONING_DRAGON = 3
        """
        All crystals (including those from the pillars) are aimed towards the
        sky. Shortly thereafter summoning the dragon and destroying the
        crystals used to initiate the dragon's respawn.
        """
        END = 4
        """
        The end of the respawn sequence. The dragon is actually summoned.
        """
        NONE = 5
        """
        No respawn is in progress.
        """



"""
Python module generated from Java source file org.bukkit.event.entity.EntityShootBowEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Entity
from org.bukkit.entity import LivingEntity
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from org.bukkit.inventory import EquipmentSlot
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class EntityShootBowEvent(EntityEvent, Cancellable):
    """
    Called when a LivingEntity shoots a bow firing an arrow
    """

    def __init__(self, shooter: "LivingEntity", bow: "ItemStack", consumable: "ItemStack", projectile: "Entity", hand: "EquipmentSlot", force: float, consumeItem: bool):
        ...


    def getEntity(self) -> "LivingEntity":
        ...


    def getBow(self) -> "ItemStack":
        """
        Gets the bow ItemStack used to fire the arrow.

        Returns
        - the bow involved in this event
        """
        ...


    def getConsumable(self) -> "ItemStack":
        """
        Get the ItemStack to be consumed in this event (if any).
        
        For instance, bows will consume an arrow ItemStack in a player's
        inventory.

        Returns
        - the consumable item
        """
        ...


    def getProjectile(self) -> "Entity":
        """
        Gets the projectile which will be launched by this event

        Returns
        - the launched projectile
        """
        ...


    def setProjectile(self, projectile: "Entity") -> None:
        """
        Replaces the projectile which will be launched

        Arguments
        - projectile: the new projectile
        """
        ...


    def getHand(self) -> "EquipmentSlot":
        """
        Get the hand from which the bow was shot.

        Returns
        - the hand
        """
        ...


    def getForce(self) -> float:
        """
        Gets the force the arrow was launched with

        Returns
        - bow shooting force, up to 1.0
        """
        ...


    def setConsumeItem(self, consumeItem: bool) -> None:
        """
        Set whether or not the consumable item should be consumed in this event.
        
        If set to False, it is recommended that a call to
        Player.updateInventory() is made as the client may disagree with
        the server's decision to not consume a consumable item.
        
        This value is ignored for entities where items are not required
        (skeletons, pillagers, etc.) or with crossbows (as no item is being
        consumed).

        Arguments
        - consumeItem: whether or not to consume the item

        Deprecated
        - not currently functional
        """
        ...


    def shouldConsumeItem(self) -> bool:
        """
        Get whether or not the consumable item should be consumed in this event.

        Returns
        - True if consumed, False otherwise
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.math.BigDecimalMath

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.math import *
from java.math import BigDecimal
from java.math import RoundingMode
from typing import Any, Callable, Iterable, Tuple


class BigDecimalMath:
    """
    A class for arithmetic on BigDecimal that is not covered by its built-in methods.

    Author(s)
    - Louis Wasserman

    Since
    - 30.0
    """

    @staticmethod
    def roundToDouble(x: "BigDecimal", mode: "RoundingMode") -> float:
        """
        Returns `x`, rounded to a `double` with the specified rounding mode. If `x`
        is precisely representable as a `double`, its `double` value will be returned;
        otherwise, the rounding will choose between the two nearest representable values with `mode`.
        
        For the case of RoundingMode.HALF_DOWN, `HALF_UP`, and `HALF_EVEN`,
        infinite `double` values are considered infinitely far away. For example, 2^2000 is not
        representable as a double, but `roundToDouble(BigDecimal.valueOf(2).pow(2000), HALF_UP)`
        will return `Double.MAX_VALUE`, not `Double.POSITIVE_INFINITY`.
        
        For the case of RoundingMode.HALF_EVEN, this implementation uses the IEEE 754
        default rounding mode: if the two nearest representable values are equally near, the one with
        the least significant bit zero is chosen. (In such cases, both of the nearest representable
        values are even integers; this method returns the one that is a multiple of a greater power of
        two.)

        Raises
        - ArithmeticException: if `mode` is RoundingMode.UNNECESSARY and `x`
            is not precisely representable as a `double`

        Since
        - 30.0
        """
        ...



"""
Python module generated from Java source file java.util.concurrent.CopyOnWriteArraySet

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import AbstractSet
from java.util import Iterator
from java.util import Objects
from java.util import Spliterator
from java.util.concurrent import *
from java.util.function import Consumer
from java.util.function import Predicate
from typing import Any, Callable, Iterable, Tuple


class CopyOnWriteArraySet(AbstractSet, Serializable):
    """
    A Set that uses an internal CopyOnWriteArrayList
    for all of its operations.  Thus, it shares the same basic properties:
    
     - It is best suited for applications in which set sizes generally
          stay small, read-only operations
          vastly outnumber mutative operations, and you need
          to prevent interference among threads during traversal.
     - It is thread-safe.
     - Mutative operations (`add`, `set`, `remove`, etc.)
         are expensive since they usually entail copying the entire underlying
         array.
     - Iterators do not support the mutative `remove` operation.
     - Traversal via iterators is fast and cannot encounter
         interference from other threads. Iterators rely on
         unchanging snapshots of the array at the time the iterators were
         constructed.
    
    
    **Sample Usage.** The following code sketch uses a
    copy-on-write set to maintain a set of Handler objects that
    perform some action upon state updates.
    
    ``` `class Handler { void handle() { ...` }
    
    class X {
      private final CopyOnWriteArraySet<Handler> handlers
        = new CopyOnWriteArraySet<>();
      public void addHandler(Handler h) { handlers.add(h); }
    
      private long internalState;
      private synchronized void changeState() { internalState = ...; }
    
      public void update() {
        changeState();
        for (Handler handler : handlers)
          handler.handle();
      }
    }}```
    
    This class is a member of the
    <a href="/java.base/java/util/package-summary.html#CollectionsFramework">
    Java Collections Framework</a>.
    
    Type `<E>`: the type of elements held in this set

    Author(s)
    - Doug Lea

    See
    - CopyOnWriteArrayList

    Since
    - 1.5
    """

    def __init__(self):
        """
        Creates an empty set.
        """
        ...


    def __init__(self, c: Iterable["E"]):
        """
        Creates a set containing all of the elements of the specified
        collection.

        Arguments
        - c: the collection of elements to initially contain

        Raises
        - NullPointerException: if the specified collection is null
        """
        ...


    def size(self) -> int:
        """
        Returns the number of elements in this set.

        Returns
        - the number of elements in this set
        """
        ...


    def isEmpty(self) -> bool:
        """
        Returns `True` if this set contains no elements.

        Returns
        - `True` if this set contains no elements
        """
        ...


    def contains(self, o: "Object") -> bool:
        """
        Returns `True` if this set contains the specified element.
        More formally, returns `True` if and only if this set
        contains an element `e` such that `Objects.equals(o, e)`.

        Arguments
        - o: element whose presence in this set is to be tested

        Returns
        - `True` if this set contains the specified element
        """
        ...


    def toArray(self) -> list["Object"]:
        """
        Returns an array containing all of the elements in this set.
        If this set makes any guarantees as to what order its elements
        are returned by its iterator, this method must return the
        elements in the same order.
        
        The returned array will be "safe" in that no references to it
        are maintained by this set.  (In other words, this method must
        allocate a new array even if this set is backed by an array).
        The caller is thus free to modify the returned array.
        
        This method acts as bridge between array-based and collection-based
        APIs.

        Returns
        - an array containing all the elements in this set
        """
        ...


    def toArray(self, a: list["T"]) -> list["T"]:
        """
        Returns an array containing all of the elements in this set; the
        runtime type of the returned array is that of the specified array.
        If the set fits in the specified array, it is returned therein.
        Otherwise, a new array is allocated with the runtime type of the
        specified array and the size of this set.
        
        If this set fits in the specified array with room to spare
        (i.e., the array has more elements than this set), the element in
        the array immediately following the end of the set is set to
        `null`.  (This is useful in determining the length of this
        set *only* if the caller knows that this set does not contain
        any null elements.)
        
        If this set makes any guarantees as to what order its elements
        are returned by its iterator, this method must return the elements
        in the same order.
        
        Like the .toArray() method, this method acts as bridge between
        array-based and collection-based APIs.  Further, this method allows
        precise control over the runtime type of the output array, and may,
        under certain circumstances, be used to save allocation costs.
        
        Suppose `x` is a set known to contain only strings.
        The following code can be used to dump the set into a newly allocated
        array of `String`:
        
        ``` `String[] y = x.toArray(new String[0]);````
        
        Note that `toArray(new Object[0])` is identical in function to
        `toArray()`.

        Arguments
        - a: the array into which the elements of this set are to be
               stored, if it is big enough; otherwise, a new array of the same
               runtime type is allocated for this purpose.

        Returns
        - an array containing all the elements in this set

        Raises
        - ArrayStoreException: if the runtime type of the specified array
                is not a supertype of the runtime type of every element in this
                set
        - NullPointerException: if the specified array is null
        """
        ...


    def clear(self) -> None:
        """
        Removes all of the elements from this set.
        The set will be empty after this call returns.
        """
        ...


    def remove(self, o: "Object") -> bool:
        """
        Removes the specified element from this set if it is present.
        More formally, removes an element `e` such that
        `Objects.equals(o, e)`, if this set contains such an element.
        Returns `True` if this set contained the element (or
        equivalently, if this set changed as a result of the call).
        (This set will not contain the element once the call returns.)

        Arguments
        - o: object to be removed from this set, if present

        Returns
        - `True` if this set contained the specified element
        """
        ...


    def add(self, e: "E") -> bool:
        """
        Adds the specified element to this set if it is not already present.
        More formally, adds the specified element `e` to this set if
        the set contains no element `e2` such that
        `Objects.equals(e, e2)`.
        If this set already contains the element, the call leaves the set
        unchanged and returns `False`.

        Arguments
        - e: element to be added to this set

        Returns
        - `True` if this set did not already contain the specified
                element
        """
        ...


    def containsAll(self, c: Iterable[Any]) -> bool:
        """
        Returns `True` if this set contains all of the elements of the
        specified collection.  If the specified collection is also a set, this
        method returns `True` if it is a *subset* of this set.

        Arguments
        - c: collection to be checked for containment in this set

        Returns
        - `True` if this set contains all of the elements of the
                specified collection

        Raises
        - NullPointerException: if the specified collection is null

        See
        - .contains(Object)
        """
        ...


    def addAll(self, c: Iterable["E"]) -> bool:
        """
        Adds all of the elements in the specified collection to this set if
        they're not already present.  If the specified collection is also a
        set, the `addAll` operation effectively modifies this set so
        that its value is the *union* of the two sets.  The behavior of
        this operation is undefined if the specified collection is modified
        while the operation is in progress.

        Arguments
        - c: collection containing elements to be added to this set

        Returns
        - `True` if this set changed as a result of the call

        Raises
        - NullPointerException: if the specified collection is null

        See
        - .add(Object)
        """
        ...


    def removeAll(self, c: Iterable[Any]) -> bool:
        """
        Removes from this set all of its elements that are contained in the
        specified collection.  If the specified collection is also a set,
        this operation effectively modifies this set so that its value is the
        *asymmetric set difference* of the two sets.

        Arguments
        - c: collection containing elements to be removed from this set

        Returns
        - `True` if this set changed as a result of the call

        Raises
        - ClassCastException: if the class of an element of this set
                is incompatible with the specified collection
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        - NullPointerException: if this set contains a null element and the
                specified collection does not permit null elements
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>),
                or if the specified collection is null

        See
        - .remove(Object)
        """
        ...


    def retainAll(self, c: Iterable[Any]) -> bool:
        """
        Retains only the elements in this set that are contained in the
        specified collection.  In other words, removes from this set all of
        its elements that are not contained in the specified collection.  If
        the specified collection is also a set, this operation effectively
        modifies this set so that its value is the *intersection* of the
        two sets.

        Arguments
        - c: collection containing elements to be retained in this set

        Returns
        - `True` if this set changed as a result of the call

        Raises
        - ClassCastException: if the class of an element of this set
                is incompatible with the specified collection
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>)
        - NullPointerException: if this set contains a null element and the
                specified collection does not permit null elements
        (<a href="/java.base/java/util/Collection.html#optional-restrictions">optional</a>),
                or if the specified collection is null

        See
        - .remove(Object)
        """
        ...


    def iterator(self) -> Iterator["E"]:
        """
        Returns an iterator over the elements contained in this set
        in the order in which these elements were added.
        
        The returned iterator provides a snapshot of the state of the set
        when the iterator was constructed. No synchronization is needed while
        traversing the iterator. The iterator does *NOT* support the
        `remove` method.

        Returns
        - an iterator over the elements in this set
        """
        ...


    def equals(self, o: "Object") -> bool:
        """
        Compares the specified object with this set for equality.
        Returns `True` if the specified object is the same object
        as this object, or if it is also a Set and the elements
        returned by an Set.iterator() iterator over the
        specified set are the same as the elements returned by an
        iterator over this set.  More formally, the two iterators are
        considered to return the same elements if they return the same
        number of elements and for every element `e1` returned by
        the iterator over the specified set, there is an element
        `e2` returned by the iterator over this set such that
        `Objects.equals(e1, e2)`.

        Arguments
        - o: object to be compared for equality with this set

        Returns
        - `True` if the specified object is equal to this set
        """
        ...


    def removeIf(self, filter: "Predicate"["E"]) -> bool:
        """
        Raises
        - NullPointerException: 
        """
        ...


    def forEach(self, action: "Consumer"["E"]) -> None:
        """
        Raises
        - NullPointerException: 
        """
        ...


    def spliterator(self) -> "Spliterator"["E"]:
        """
        Returns a Spliterator over the elements in this set in the order
        in which these elements were added.
        
        The `Spliterator` reports Spliterator.IMMUTABLE,
        Spliterator.DISTINCT, Spliterator.SIZED, and
        Spliterator.SUBSIZED.
        
        The spliterator provides a snapshot of the state of the set
        when the spliterator was constructed. No synchronization is needed while
        operating on the spliterator.

        Returns
        - a `Spliterator` over the elements in this set

        Since
        - 1.8
        """
        ...



"""
Python module generated from Java source file org.bukkit.metadata.LazyMetadataValue

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from enum import Enum
from java.lang.ref import SoftReference
from java.util.concurrent import Callable
from org.bukkit.metadata import *
from org.bukkit.plugin import Plugin
from typing import Any, Callable, Iterable, Tuple


class LazyMetadataValue(MetadataValueAdapter):
    """
    The LazyMetadataValue class implements a type of metadata that is not
    computed until another plugin asks for it.
    
    By making metadata values lazy, no computation is done by the providing
    plugin until absolutely necessary (if ever). Additionally,
    LazyMetadataValue objects cache their values internally unless overridden
    by a CacheStrategy or invalidated at the individual or plugin
    level. Once invalidated, the LazyMetadataValue will recompute its value
    when asked.
    """

    def __init__(self, owningPlugin: "Plugin", lazyValue: "Callable"["Object"]):
        """
        Initialized a LazyMetadataValue object with the default
        CACHE_AFTER_FIRST_EVAL cache strategy.

        Arguments
        - owningPlugin: the Plugin that created this metadata
            value.
        - lazyValue: the lazy value assigned to this metadata value.
        """
        ...


    def __init__(self, owningPlugin: "Plugin", cacheStrategy: "CacheStrategy", lazyValue: "Callable"["Object"]):
        """
        Initializes a LazyMetadataValue object with a specific cache strategy.

        Arguments
        - owningPlugin: the Plugin that created this metadata
            value.
        - cacheStrategy: determines the rules for caching this metadata
            value.
        - lazyValue: the lazy value assigned to this metadata value.
        """
        ...


    def value(self) -> "Object":
        ...


    def invalidate(self) -> None:
        ...


    class CacheStrategy(Enum):
        """
        Describes possible caching strategies for metadata.
        """

        CACHE_AFTER_FIRST_EVAL = 0
        """
        Once the metadata value has been evaluated, do not re-evaluate the
        value until it is manually invalidated.
        """
        NEVER_CACHE = 1
        """
        Re-evaluate the metadata item every time it is requested
        """
        CACHE_ETERNALLY = 2
        """
        Once the metadata value has been evaluated, do not re-evaluate the
        value in spite of manual invalidation.
        """



"""
Python module generated from Java source file com.google.common.base.Supplier

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import *
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Supplier(Supplier):
    """
    Legacy version of java.util.function.Supplier java.util.function.Supplier. Semantically,
    this could be a factory, generator, builder, closure, or something else entirely. No guarantees
    are implied by this interface.
    
    The Suppliers class provides common suppliers and related utilities.
    
    As this interface extends `java.util.function.Supplier`, an instance of this type can be
    used as a `java.util.function.Supplier` directly. To use a `java.util.function.Supplier` in a context where a `com.google.common.base.Supplier` is
    needed, use `supplier::get`.
    
    See the Guava User Guide article on <a
    href="https://github.com/google/guava/wiki/FunctionalExplained">the use of `Function`</a>.

    Author(s)
    - Harry Heymann

    Since
    - 2.0
    """

    def get(self) -> "T":
        """
        Retrieves an instance of the appropriate type. The returned object may or may not be a new
        instance, depending on the implementation.

        Returns
        - an instance of the appropriate type
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.meta.AxolotlBucketMeta

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Axolotl
from org.bukkit.inventory.meta import *
from typing import Any, Callable, Iterable, Tuple


class AxolotlBucketMeta(ItemMeta):
    """
    Represents a bucket of axolotl.
    """

    def getVariant(self) -> "Axolotl.Variant":
        """
        Get the variant of the axolotl in the bucket.
        
        Plugins should check that hasVariant() returns `True` before
        calling this method.

        Returns
        - axolotl variant
        """
        ...


    def setVariant(self, variant: "Axolotl.Variant") -> None:
        """
        Set the variant of this axolotl in the bucket.

        Arguments
        - variant: axolotl variant
        """
        ...


    def hasVariant(self) -> bool:
        """
        Checks for existence of a variant tag indicating a specific axolotl will be
        spawned.

        Returns
        - if there is a variant
        """
        ...


    def clone(self) -> "AxolotlBucketMeta":
        ...



"""
Python module generated from Java source file org.bukkit.OfflinePlayer

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.time import Duration
from java.time import Instant
from java.util import Date
from java.util import UUID
from org.bukkit import *
from org.bukkit.ban import ProfileBanList
from org.bukkit.configuration.serialization import ConfigurationSerializable
from org.bukkit.entity import AnimalTamer
from org.bukkit.entity import EntityType
from org.bukkit.entity import Player
from org.bukkit.permissions import ServerOperator
from org.bukkit.profile import PlayerProfile
from typing import Any, Callable, Iterable, Tuple


class OfflinePlayer(ServerOperator, AnimalTamer, ConfigurationSerializable):
    """
    Represents a reference to a player identity and the data belonging to a
    player that is stored on the disk and can, thus, be retrieved without the
    player needing to be online.
    """

    def isOnline(self) -> bool:
        """
        Checks if this player is currently online

        Returns
        - True if they are online
        """
        ...


    def getName(self) -> str:
        """
        Returns the name of this player
        
        Names are no longer unique past a single game session. For persistent storage
        it is recommended that you use .getUniqueId() instead.

        Returns
        - Player name or null if we have not seen a name for this player yet
        """
        ...


    def getUniqueId(self) -> "UUID":
        """
        Returns the UUID of this player

        Returns
        - Player UUID
        """
        ...


    def getPlayerProfile(self) -> "PlayerProfile":
        """
        Gets a copy of the player's profile.
        
        If the player is online, the returned profile will be complete.
        Otherwise, only the unique id is guaranteed to be present. You can use
        PlayerProfile.update() to complete the returned profile.

        Returns
        - the player's profile
        """
        ...


    def isBanned(self) -> bool:
        """
        Checks if this player has had their profile banned.

        Returns
        - True if banned, otherwise False
        """
        ...


    def ban(self, reason: str, expires: "Date", source: str) -> "BanEntry"["PlayerProfile"]:
        """
        Adds this user to the ProfileBanList. If a previous ban exists, this will
        update the entry.

        Arguments
        - reason: reason for the ban, null indicates implementation default
        - expires: date for the ban's expiration (unban), or null to imply
            forever
        - source: source of the ban, null indicates implementation default

        Returns
        - the entry for the newly created ban, or the entry for the
            (updated) previous ban
        """
        ...


    def ban(self, reason: str, expires: "Instant", source: str) -> "BanEntry"["PlayerProfile"]:
        """
        Adds this user to the ProfileBanList. If a previous ban exists, this will
        update the entry.

        Arguments
        - reason: reason for the ban, null indicates implementation default
        - expires: instant for the ban's expiration (unban), or null to imply
            forever
        - source: source of the ban, null indicates implementation default

        Returns
        - the entry for the newly created ban, or the entry for the
            (updated) previous ban
        """
        ...


    def ban(self, reason: str, duration: "Duration", source: str) -> "BanEntry"["PlayerProfile"]:
        """
        Adds this user to the ProfileBanList. If a previous ban exists, this will
        update the entry.

        Arguments
        - reason: reason for the ban, null indicates implementation default
        - duration: how long the ban last, or null to imply
            forever
        - source: source of the ban, null indicates implementation default

        Returns
        - the entry for the newly created ban, or the entry for the
            (updated) previous ban
        """
        ...


    def isWhitelisted(self) -> bool:
        """
        Checks if this player is whitelisted or not

        Returns
        - True if whitelisted
        """
        ...


    def setWhitelisted(self, value: bool) -> None:
        """
        Sets if this player is whitelisted or not

        Arguments
        - value: True if whitelisted
        """
        ...


    def getPlayer(self) -> "Player":
        """
        Gets a Player object that this represents, if there is one
        
        If the player is online, this will return that player. Otherwise,
        it will return null.

        Returns
        - Online player
        """
        ...


    def getFirstPlayed(self) -> int:
        """
        Gets the first date and time that this player was witnessed on this
        server.
        
        If the player has never played before, this will return 0. Otherwise,
        it will be the amount of milliseconds since midnight, January 1, 1970
        UTC.

        Returns
        - Date of first log-in for this player, or 0
        """
        ...


    def getLastPlayed(self) -> int:
        """
        Gets the last date and time that this player was witnessed on this
        server.
        
        If the player has never played before, this will return 0. Otherwise,
        it will be the amount of milliseconds since midnight, January 1, 1970
        UTC.

        Returns
        - Date of last log-in for this player, or 0
        """
        ...


    def hasPlayedBefore(self) -> bool:
        """
        Checks if this player has played on this server before.

        Returns
        - True if the player has played before, otherwise False
        """
        ...


    def getBedSpawnLocation(self) -> "Location":
        """
        Gets the Location where the player will spawn at their bed, null if
        they have not slept in one or their current bed spawn is invalid.

        Returns
        - Bed Spawn Location if bed exists, otherwise null.

        See
        - .getRespawnLocation()

        Deprecated
        - Misleading name. This method also returns the location of
        respawn anchors.
        """
        ...


    def getRespawnLocation(self) -> "Location":
        """
        Gets the Location where the player will spawn at, null if they
        don't have a valid respawn point.

        Returns
        - respawn location if exists, otherwise null.
        """
        ...


    def incrementStatistic(self, statistic: "Statistic") -> None:
        """
        Increments the given statistic for this player.
        
        This is equivalent to the following code:
        `incrementStatistic(Statistic, 1)`

        Arguments
        - statistic: Statistic to increment

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if the statistic requires an
            additional parameter
        """
        ...


    def decrementStatistic(self, statistic: "Statistic") -> None:
        """
        Decrements the given statistic for this player.
        
        This is equivalent to the following code:
        `decrementStatistic(Statistic, 1)`

        Arguments
        - statistic: Statistic to decrement

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if the statistic requires an
            additional parameter
        """
        ...


    def incrementStatistic(self, statistic: "Statistic", amount: int) -> None:
        """
        Increments the given statistic for this player.

        Arguments
        - statistic: Statistic to increment
        - amount: Amount to increment this statistic by

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if amount is negative
        - IllegalArgumentException: if the statistic requires an
            additional parameter
        """
        ...


    def decrementStatistic(self, statistic: "Statistic", amount: int) -> None:
        """
        Decrements the given statistic for this player.

        Arguments
        - statistic: Statistic to decrement
        - amount: Amount to decrement this statistic by

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if amount is negative
        - IllegalArgumentException: if the statistic requires an
            additional parameter
        """
        ...


    def setStatistic(self, statistic: "Statistic", newValue: int) -> None:
        """
        Sets the given statistic for this player.

        Arguments
        - statistic: Statistic to set
        - newValue: The value to set this statistic to

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if newValue is negative
        - IllegalArgumentException: if the statistic requires an
            additional parameter
        """
        ...


    def getStatistic(self, statistic: "Statistic") -> int:
        """
        Gets the value of the given statistic for this player.

        Arguments
        - statistic: Statistic to check

        Returns
        - the value of the given statistic

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if the statistic requires an
            additional parameter
        """
        ...


    def incrementStatistic(self, statistic: "Statistic", material: "Material") -> None:
        """
        Increments the given statistic for this player for the given material.
        
        This is equivalent to the following code:
        `incrementStatistic(Statistic, Material, 1)`

        Arguments
        - statistic: Statistic to increment
        - material: Material to offset the statistic with

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if material is null
        - IllegalArgumentException: if the given parameter is not valid
            for the statistic
        """
        ...


    def decrementStatistic(self, statistic: "Statistic", material: "Material") -> None:
        """
        Decrements the given statistic for this player for the given material.
        
        This is equivalent to the following code:
        `decrementStatistic(Statistic, Material, 1)`

        Arguments
        - statistic: Statistic to decrement
        - material: Material to offset the statistic with

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if material is null
        - IllegalArgumentException: if the given parameter is not valid
            for the statistic
        """
        ...


    def getStatistic(self, statistic: "Statistic", material: "Material") -> int:
        """
        Gets the value of the given statistic for this player.

        Arguments
        - statistic: Statistic to check
        - material: Material offset of the statistic

        Returns
        - the value of the given statistic

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if material is null
        - IllegalArgumentException: if the given parameter is not valid
            for the statistic
        """
        ...


    def incrementStatistic(self, statistic: "Statistic", material: "Material", amount: int) -> None:
        """
        Increments the given statistic for this player for the given material.

        Arguments
        - statistic: Statistic to increment
        - material: Material to offset the statistic with
        - amount: Amount to increment this statistic by

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if material is null
        - IllegalArgumentException: if amount is negative
        - IllegalArgumentException: if the given parameter is not valid
            for the statistic
        """
        ...


    def decrementStatistic(self, statistic: "Statistic", material: "Material", amount: int) -> None:
        """
        Decrements the given statistic for this player for the given material.

        Arguments
        - statistic: Statistic to decrement
        - material: Material to offset the statistic with
        - amount: Amount to decrement this statistic by

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if material is null
        - IllegalArgumentException: if amount is negative
        - IllegalArgumentException: if the given parameter is not valid
            for the statistic
        """
        ...


    def setStatistic(self, statistic: "Statistic", material: "Material", newValue: int) -> None:
        """
        Sets the given statistic for this player for the given material.

        Arguments
        - statistic: Statistic to set
        - material: Material to offset the statistic with
        - newValue: The value to set this statistic to

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if material is null
        - IllegalArgumentException: if newValue is negative
        - IllegalArgumentException: if the given parameter is not valid
            for the statistic
        """
        ...


    def incrementStatistic(self, statistic: "Statistic", entityType: "EntityType") -> None:
        """
        Increments the given statistic for this player for the given entity.
        
        This is equivalent to the following code:
        `incrementStatistic(Statistic, EntityType, 1)`

        Arguments
        - statistic: Statistic to increment
        - entityType: EntityType to offset the statistic with

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if entityType is null
        - IllegalArgumentException: if the given parameter is not valid
            for the statistic
        """
        ...


    def decrementStatistic(self, statistic: "Statistic", entityType: "EntityType") -> None:
        """
        Decrements the given statistic for this player for the given entity.
        
        This is equivalent to the following code:
        `decrementStatistic(Statistic, EntityType, 1)`

        Arguments
        - statistic: Statistic to decrement
        - entityType: EntityType to offset the statistic with

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if entityType is null
        - IllegalArgumentException: if the given parameter is not valid
            for the statistic
        """
        ...


    def getStatistic(self, statistic: "Statistic", entityType: "EntityType") -> int:
        """
        Gets the value of the given statistic for this player.

        Arguments
        - statistic: Statistic to check
        - entityType: EntityType offset of the statistic

        Returns
        - the value of the given statistic

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if entityType is null
        - IllegalArgumentException: if the given parameter is not valid
            for the statistic
        """
        ...


    def incrementStatistic(self, statistic: "Statistic", entityType: "EntityType", amount: int) -> None:
        """
        Increments the given statistic for this player for the given entity.

        Arguments
        - statistic: Statistic to increment
        - entityType: EntityType to offset the statistic with
        - amount: Amount to increment this statistic by

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if entityType is null
        - IllegalArgumentException: if amount is negative
        - IllegalArgumentException: if the given parameter is not valid
            for the statistic
        """
        ...


    def decrementStatistic(self, statistic: "Statistic", entityType: "EntityType", amount: int) -> None:
        """
        Decrements the given statistic for this player for the given entity.

        Arguments
        - statistic: Statistic to decrement
        - entityType: EntityType to offset the statistic with
        - amount: Amount to decrement this statistic by

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if entityType is null
        - IllegalArgumentException: if amount is negative
        - IllegalArgumentException: if the given parameter is not valid
            for the statistic
        """
        ...


    def setStatistic(self, statistic: "Statistic", entityType: "EntityType", newValue: int) -> None:
        """
        Sets the given statistic for this player for the given entity.

        Arguments
        - statistic: Statistic to set
        - entityType: EntityType to offset the statistic with
        - newValue: The value to set this statistic to

        Raises
        - IllegalArgumentException: if statistic is null
        - IllegalArgumentException: if entityType is null
        - IllegalArgumentException: if newValue is negative
        - IllegalArgumentException: if the given parameter is not valid
            for the statistic
        """
        ...


    def getLastDeathLocation(self) -> "Location":
        """
        Gets the player's last death location.

        Returns
        - the last death location if it exists, otherwise null.
        """
        ...


    def getLocation(self) -> "Location":
        """
        Gets the player's current location.

        Returns
        - the player's location, `null` if player hasn't ever played
        before.
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.block.TNTPrimeEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.block import Block
from org.bukkit.entity import Entity
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.block import *
from typing import Any, Callable, Iterable, Tuple


class TNTPrimeEvent(BlockEvent, Cancellable):
    """
    Called when a block of TNT in the world become primed.
    
    If a TNT Prime event is cancelled, the block of TNT will not become primed.
    """

    def __init__(self, block: "Block", igniteCause: "PrimeCause", primingEntity: "Entity", primingBlock: "Block"):
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getCause(self) -> "PrimeCause":
        """
        Get the cause of the TNT becoming primed.

        Returns
        - the cause
        """
        ...


    def getPrimingEntity(self) -> "Entity":
        """
        Get the entity that caused the TNT to be primed.

        Returns
        - the entity that caused the TNT to be primed, or null if it was
        not caused by an entity.
        """
        ...


    def getPrimingBlock(self) -> "Block":
        """
        Get the block that caused the TNT to be primed.

        Returns
        - the block that caused the TNT to be primed, or null if it was not
        caused by a block.
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...


    class PrimeCause(Enum):
        """
        An enum to represent the cause of a TNT block becoming primed.
        """

        FIRE = 0
        """
        When TNT is primed by fire spreading.
        """
        REDSTONE = 1
        """
        When TNT is primed by a redstone signal.
        """
        PLAYER = 2
        """
        When TNT is primed by a player interacting with it directly.
        """
        EXPLOSION = 3
        """
        When TNT is primed by a nearby explosion.
        """
        PROJECTILE = 4
        """
        When TNT is primed after getting hit with a burning projectile.
        """
        BLOCK_BREAK = 5
        """
        When TNT with the unstable block state set to True is broken.
        
        Note: Canceling a prime event with this cause will stop the primed
        TNT from spawning but will not stop the block from being broken.
        """
        DISPENSER = 6
        """
        When TNT is primed by a dispenser holding flint and steel.
        
        Note: This event is not called for a dispenser dispensing TNT
        directly.
        """



"""
Python module generated from Java source file org.bukkit.FeatureFlag

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class FeatureFlag(Keyed):
    """
    This represents a Feature Flag for a World.
    
    Flags which are unavailable in the current version will be null and/or
    removed.
    """

    VANILLA = Bukkit.getUnsafe().getFeatureFlag(NamespacedKey.minecraft("vanilla"))
    BUNDLE = Bukkit.getUnsafe().getFeatureFlag(NamespacedKey.minecraft("bundle"))
    """
    <strong>AVAILABLE BETWEEN VERSIONS:</strong> 1.19.3 - 1.21.1

    Deprecated
    - not available since 1.21.2
    """
    UPDATE_1_20 = Bukkit.getUnsafe().getFeatureFlag(NamespacedKey.minecraft("update_1_20"))
    """
    <strong>AVAILABLE BETWEEN VERSIONS:</strong> 1.19 - 1.19.4

    Deprecated
    - not available since 1.20
    """
    TRADE_REBALANCE = Bukkit.getUnsafe().getFeatureFlag(NamespacedKey.minecraft("trade_rebalance"))
    UPDATE_121 = Bukkit.getUnsafe().getFeatureFlag(NamespacedKey.minecraft("update_1_21"))
    """
    <strong>AVAILABLE BETWEEN VERSIONS:</strong> 1.20.5 - 1.20.6

    Deprecated
    - not available since 1.21
    """
    WINTER_DROP = Bukkit.getUnsafe().getFeatureFlag(NamespacedKey.minecraft("winter_drop"))
    """
    <strong>AVAILABLE BETWEEN VERSIONS:</strong> 1.21.2 - 1.21.3

    Deprecated
    - not available since 1.21.4
    """
    REDSTONE_EXPERIMENTS = Bukkit.getUnsafe().getFeatureFlag(NamespacedKey.minecraft("redstone_experiments"))
    MINECART_IMPROVEMENTS = Bukkit.getUnsafe().getFeatureFlag(NamespacedKey.minecraft("minecart_improvements"))



"""
Python module generated from Java source file org.bukkit.command.FormattedCommandAlias

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Bukkit
from org.bukkit.command import *
from typing import Any, Callable, Iterable, Tuple


class FormattedCommandAlias(Command):

    def __init__(self, alias: str, formatStrings: list[str]):
        ...


    def execute(self, sender: "CommandSender", commandLabel: str, args: list[str]) -> bool:
        ...



"""
Python module generated from Java source file java.util.concurrent.Future

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.concurrent import *
from typing import Any, Callable, Iterable, Tuple


class Future:
    """
    A `Future` represents the result of an asynchronous
    computation.  Methods are provided to check if the computation is
    complete, to wait for its completion, and to retrieve the result of
    the computation.  The result can only be retrieved using method
    `get` when the computation has completed, blocking if
    necessary until it is ready.  Cancellation is performed by the
    `cancel` method.  Additional methods are provided to
    determine if the task completed normally or was cancelled. Once a
    computation has completed, the computation cannot be cancelled.
    If you would like to use a `Future` for the sake
    of cancellability but not provide a usable result, you can
    declare types of the form `Future<?>` and
    return `null` as a result of the underlying task.
    
    **Sample Usage** (Note that the following classes are all
    made-up.)
    
    ``` `interface ArchiveSearcher { String search(String target);`
    class App {
      ExecutorService executor = ...;
      ArchiveSearcher searcher = ...;
      void showSearch(String target) throws InterruptedException {
        Callable<String> task = () -> searcher.search(target);
        Future<String> future = executor.submit(task);
        displayOtherThings(); // do other things while searching
        try {
          displayText(future.get()); // use future
        } catch (ExecutionException ex) { cleanup(); return; }
      }
    }}```
    
    The FutureTask class is an implementation of `Future` that
    implements `Runnable`, and so may be executed by an `Executor`.
    For example, the above construction with `submit` could be replaced by:
    ``` `FutureTask<String> future = new FutureTask<>(task);
    executor.execute(future);````
    
    Memory consistency effects: Actions taken by the asynchronous computation
    <a href="package-summary.html#MemoryVisibility"> *happen-before*</a>
    actions following the corresponding `Future.get()` in another thread.
    
    Type `<V>`: The result type returned by this Future's `get` method

    Author(s)
    - Doug Lea

    See
    - Executor

    Since
    - 1.5
    """

    def cancel(self, mayInterruptIfRunning: bool) -> bool:
        """
        Attempts to cancel execution of this task.  This method has no
        effect if the task is already completed or cancelled, or could
        not be cancelled for some other reason.  Otherwise, if this
        task has not started when `cancel` is called, this task
        should never run.  If the task has already started, then the
        `mayInterruptIfRunning` parameter determines whether the
        thread executing this task (when known by the implementation)
        is interrupted in an attempt to stop the task.
        
        The return value from this method does not necessarily
        indicate whether the task is now cancelled; use .isCancelled.

        Arguments
        - mayInterruptIfRunning: `True` if the thread
        executing this task should be interrupted (if the thread is
        known to the implementation); otherwise, in-progress tasks are
        allowed to complete

        Returns
        - `False` if the task could not be cancelled,
        typically because it has already completed; `True`
        otherwise. If two or more threads cause a task to be cancelled,
        then at least one of them returns `True`. Implementations
        may provide stronger guarantees.
        """
        ...


    def isCancelled(self) -> bool:
        """
        Returns `True` if this task was cancelled before it completed
        normally.

        Returns
        - `True` if this task was cancelled before it completed
        """
        ...


    def isDone(self) -> bool:
        """
        Returns `True` if this task completed.
        
        Completion may be due to normal termination, an exception, or
        cancellation -- in all of these cases, this method will return
        `True`.

        Returns
        - `True` if this task completed
        """
        ...


    def get(self) -> "V":
        """
        Waits if necessary for the computation to complete, and then
        retrieves its result.

        Returns
        - the computed result

        Raises
        - CancellationException: if the computation was cancelled
        - ExecutionException: if the computation threw an
        exception
        - InterruptedException: if the current thread was interrupted
        while waiting
        """
        ...


    def get(self, timeout: int, unit: "TimeUnit") -> "V":
        """
        Waits if necessary for at most the given time for the computation
        to complete, and then retrieves its result, if available.

        Arguments
        - timeout: the maximum time to wait
        - unit: the time unit of the timeout argument

        Returns
        - the computed result

        Raises
        - CancellationException: if the computation was cancelled
        - ExecutionException: if the computation threw an
        exception
        - InterruptedException: if the current thread was interrupted
        while waiting
        - TimeoutException: if the wait timed out
        """
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.ForwardingFuture

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import Preconditions
from com.google.common.collect import ForwardingObject
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.util.concurrent import ExecutionException
from java.util.concurrent import Future
from java.util.concurrent import TimeUnit
from java.util.concurrent import TimeoutException
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ForwardingFuture(ForwardingObject, Future):
    """
    A Future which forwards all its method calls to another future. Subclasses should
    override one or more methods to modify the behavior of the backing future as desired per the <a
    href="http://en.wikipedia.org/wiki/Decorator_pattern">decorator pattern</a>.
    
    Most subclasses can just use SimpleForwardingFuture.

    Author(s)
    - Sven Mawson

    Since
    - 1.0
    """

    def cancel(self, mayInterruptIfRunning: bool) -> bool:
        ...


    def isCancelled(self) -> bool:
        ...


    def isDone(self) -> bool:
        ...


    def get(self) -> "V":
        ...


    def get(self, timeout: int, unit: "TimeUnit") -> "V":
        ...


    class SimpleForwardingFuture(ForwardingFuture):
        """
        A simplified version of ForwardingFuture where subclasses can pass in an already
        constructed Future as the delegate.

        Since
        - 9.0
        """





"""
Python module generated from Java source file org.yaml.snakeyaml.events.MappingStartEvent

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml import DumperOptions
from org.yaml.snakeyaml.error import Mark
from org.yaml.snakeyaml.events import *
from typing import Any, Callable, Iterable, Tuple


class MappingStartEvent(CollectionStartEvent):
    """
    Marks the beginning of a mapping node.
    
    This event is followed by a number of key value pairs. 
    The pairs are not in any particular order. However, the value always directly follows the
    corresponding key. 
    After the key value pairs follows a MappingEndEvent.
    
    
    There must be an even number of node events between the start and end event.

    See
    - MappingEndEvent
    """

    def __init__(self, anchor: str, tag: str, implicit: bool, startMark: "Mark", endMark: "Mark", flowStyle: "DumperOptions.FlowStyle"):
        ...


    def getEventId(self) -> "Event.ID":
        """
        getter

        Returns
        - its identity
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.Tameable

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class Tameable(Animals):

    def isTamed(self) -> bool:
        """
        Check if this is tamed
        
        If something is tamed then a player can not tame it through normal
        methods, even if it does not belong to anyone in particular.

        Returns
        - True if this has been tamed
        """
        ...


    def setTamed(self, tame: bool) -> None:
        """
        Sets if this has been tamed. Not necessary if the method setOwner has
        been used, as it tames automatically.
        
        If something is tamed then a player can not tame it through normal
        methods, even if it does not belong to anyone in particular.

        Arguments
        - tame: True if tame
        """
        ...


    def getOwner(self) -> "AnimalTamer":
        """
        Gets the current owning AnimalTamer

        Returns
        - the owning AnimalTamer, or null if not owned
        """
        ...


    def setOwner(self, tamer: "AnimalTamer") -> None:
        """
        Set this to be owned by given AnimalTamer.
        
        If the owner is not null, this will be tamed and will have any current
        path it is following removed. If the owner is set to null, this will be
        untamed, and the current owner removed.

        Arguments
        - tamer: the AnimalTamer who should own this
        """
        ...



"""
Python module generated from Java source file java.io.OutputStream

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from java.util import Objects
from typing import Any, Callable, Iterable, Tuple


class OutputStream(Closeable, Flushable):
    """
    This abstract class is the superclass of all classes representing
    an output stream of bytes. An output stream accepts output bytes
    and sends them to some sink.
    
    Applications that need to define a subclass of
    `OutputStream` must always provide at least a method
    that writes one byte of output.

    Author(s)
    - Arthur van Hoff

    See
    - java.io.OutputStream.write(int)

    Since
    - 1.0
    """

    def __init__(self):
        """
        Constructor for subclasses to call.
        """
        ...


    @staticmethod
    def nullOutputStream() -> "OutputStream":
        """
        Returns a new `OutputStream` which discards all bytes.  The
        returned stream is initially open.  The stream is closed by calling
        the `close()` method.  Subsequent calls to `close()` have
        no effect.
        
         While the stream is open, the `write(int)`, `write(byte[])`, and `write(byte[], int, int)` methods do nothing.
        After the stream has been closed, these methods all throw `IOException`.
        
         The `flush()` method does nothing.

        Returns
        - an `OutputStream` which discards all bytes

        Since
        - 11
        """
        ...


    def write(self, b: int) -> None:
        """
        Writes the specified byte to this output stream. The general
        contract for `write` is that one byte is written
        to the output stream. The byte to be written is the eight
        low-order bits of the argument `b`. The 24
        high-order bits of `b` are ignored.
        
        Subclasses of `OutputStream` must provide an
        implementation for this method.

        Arguments
        - b: the `byte`.

        Raises
        - IOException: if an I/O error occurs. In particular,
                    an `IOException` may be thrown if the
                    output stream has been closed.
        """
        ...


    def write(self, b: list[int]) -> None:
        """
        Writes `b.length` bytes from the specified byte array
        to this output stream. The general contract for `write(b)`
        is that it should have exactly the same effect as the call
        `write(b, 0, b.length)`.

        Arguments
        - b: the data.

        Raises
        - IOException: if an I/O error occurs.

        See
        - java.io.OutputStream.write(byte[], int, int)
        """
        ...


    def write(self, b: list[int], off: int, len: int) -> None:
        """
        Writes `len` bytes from the specified byte array
        starting at offset `off` to this output stream.
        The general contract for `write(b, off, len)` is that
        some of the bytes in the array `b` are written to the
        output stream in order; element `b[off]` is the first
        byte written and `b[off+len-1]` is the last byte written
        by this operation.
        
        The `write` method of `OutputStream` calls
        the write method of one argument on each of the bytes to be
        written out. Subclasses are encouraged to override this method and
        provide a more efficient implementation.
        
        If `b` is `null`, a
        `NullPointerException` is thrown.
        
        If `off` is negative, or `len` is negative, or
        `off+len` is greater than the length of the array
        `b`, then an `IndexOutOfBoundsException` is thrown.

        Arguments
        - b: the data.
        - off: the start offset in the data.
        - len: the number of bytes to write.

        Raises
        - IOException: if an I/O error occurs. In particular,
                    an `IOException` is thrown if the output
                    stream is closed.
        """
        ...


    def flush(self) -> None:
        """
        Flushes this output stream and forces any buffered output bytes
        to be written out. The general contract of `flush` is
        that calling it is an indication that, if any bytes previously
        written have been buffered by the implementation of the output
        stream, such bytes should immediately be written to their
        intended destination.
        
        If the intended destination of this stream is an abstraction provided by
        the underlying operating system, for example a file, then flushing the
        stream guarantees only that bytes previously written to the stream are
        passed to the operating system for writing; it does not guarantee that
        they are actually written to a physical device such as a disk drive.
        
        The `flush` method of `OutputStream` does nothing.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def close(self) -> None:
        """
        Closes this output stream and releases any system resources
        associated with this stream. The general contract of `close`
        is that it closes the output stream. A closed stream cannot perform
        output operations and cannot be reopened.
        
        The `close` method of `OutputStream` does nothing.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...



"""
Python module generated from Java source file com.google.common.base.Throwables

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.base import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.io import IOException
from java.io import PrintWriter
from java.io import StringWriter
from java.lang.reflect import InvocationTargetException
from java.lang.reflect import Method
from java.util import AbstractList
from java.util import Collections
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Throwables:
    """
    Static utility methods pertaining to instances of Throwable.
    
    See the Guava User Guide entry on <a
    href="https://github.com/google/guava/wiki/ThrowablesExplained">Throwables</a>.

    Author(s)
    - Ben Yu

    Since
    - 1.0
    """

    @staticmethod
    def throwIfInstanceOf(throwable: "Throwable", declaredType: type["X"]) -> None:
        """
        Throws `throwable` if it is an instance of `declaredType`. Example usage:
        
        ```
        for (Foo foo : foos) {
          try {
            foo.bar();
          } catch (BarException | RuntimeException | Error t) {
            failure = t;
          }
        }
        if (failure != null) {
          throwIfInstanceOf(failure, BarException.class);
          throwIfUnchecked(failure);
          throw new AssertionError(failure);
        }
        ```

        Since
        - 20.0
        """
        ...


    @staticmethod
    def propagateIfInstanceOf(throwable: "Throwable", declaredType: type["X"]) -> None:
        """
        Propagates `throwable` exactly as-is, if and only if it is an instance of `declaredType`. Example usage:
        
        ```
        try {
          someMethodThatCouldThrowAnything();
        } catch (IKnowWhatToDoWithThisException e) {
          handle(e);
        } catch (Throwable t) {
          Throwables.propagateIfInstanceOf(t, IOException.class);
          Throwables.propagateIfInstanceOf(t, SQLException.class);
          throw Throwables.propagate(t);
        }
        ```

        Deprecated
        - Use .throwIfInstanceOf, which has the same behavior but rejects `null`.
        """
        ...


    @staticmethod
    def throwIfUnchecked(throwable: "Throwable") -> None:
        """
        Throws `throwable` if it is a RuntimeException or Error. Example usage:
        
        ```
        for (Foo foo : foos) {
          try {
            foo.bar();
          } catch (RuntimeException | Error t) {
            failure = t;
          }
        }
        if (failure != null) {
          throwIfUnchecked(failure);
          throw new AssertionError(failure);
        }
        ```

        Since
        - 20.0
        """
        ...


    @staticmethod
    def propagateIfPossible(throwable: "Throwable") -> None:
        """
        Propagates `throwable` exactly as-is, if and only if it is an instance of RuntimeException or Error.

        Deprecated
        - Use .throwIfUnchecked, which has the same behavior but rejects `null`.
        """
        ...


    @staticmethod
    def propagateIfPossible(throwable: "Throwable", declaredType: type["X"]) -> None:
        """
        Propagates `throwable` exactly as-is, if and only if it is an instance of RuntimeException, Error, or `declaredType`.
        
        **Discouraged** in favor of calling .throwIfInstanceOf and .throwIfUnchecked.

        Arguments
        - throwable: the Throwable to possibly propagate
        - declaredType: the single checked exception type declared by the calling method

        Deprecated
        - Use a combination of .throwIfInstanceOf and .throwIfUnchecked,
            which togther provide the same behavior except that they reject `null`.
        """
        ...


    @staticmethod
    def propagateIfPossible(throwable: "Throwable", declaredType1: type["X1"], declaredType2: type["X2"]) -> None:
        """
        Propagates `throwable` exactly as-is, if and only if it is an instance of RuntimeException, Error, `declaredType1`, or `declaredType2`.

        Arguments
        - throwable: the Throwable to possibly propagate
        - declaredType1: any checked exception type declared by the calling method
        - declaredType2: any other checked exception type declared by the calling method

        Deprecated
        - Use a combination of two calls to .throwIfInstanceOf and one call to .throwIfUnchecked, which togther provide the same behavior except that they reject `null`.
        """
        ...


    @staticmethod
    def propagate(throwable: "Throwable") -> "RuntimeException":
        """
        Propagates `throwable` as-is if it is an instance of RuntimeException or Error, or else as a last resort, wraps it in a `RuntimeException` and then propagates.
        
        This method always throws an exception. The `RuntimeException` return type allows
        client code to signal to the compiler that statements after the call are unreachable. Example
        usage:
        
        ```
        T doSomething() {
          try {
            return someMethodThatCouldThrowAnything();
          } catch (IKnowWhatToDoWithThisException e) {
            return handle(e);
          } catch (Throwable t) {
            throw Throwables.propagate(t);
          }
        }
        ```

        Arguments
        - throwable: the Throwable to propagate

        Returns
        - nothing will ever be returned; this return type is only for your convenience, as
            illustrated in the example above

        Deprecated
        - To preserve behavior, use `throw e` or `throw new RuntimeException(e)`
            directly, or use a combination of .throwIfUnchecked and `throw new
            RuntimeException(e)`. But consider whether users would be better off if your API threw a
            different type of exception. For background on the deprecation, read <a
            href="https://github.com/google/guava/wiki/Why-we-deprecated-Throwables.propagate">Why we
            deprecated `Throwables.propagate`</a>.
        """
        ...


    @staticmethod
    def getRootCause(throwable: "Throwable") -> "Throwable":
        """
        Returns the innermost cause of `throwable`. The first throwable in a chain provides
        context from when the error or exception was initially detected. Example usage:
        
        ```
        assertEquals("Unable to assign a customer id", Throwables.getRootCause(e).getMessage());
        ```

        Raises
        - IllegalArgumentException: if there is a loop in the causal chain
        """
        ...


    @staticmethod
    def getCausalChain(throwable: "Throwable") -> list["Throwable"]:
        """
        Gets a `Throwable` cause chain as a list. The first entry in the list will be `throwable` followed by its cause hierarchy. Note that this is a snapshot of the cause chain and
        will not reflect any subsequent changes to the cause chain.
        
        Here's an example of how it can be used to find specific types of exceptions in the cause
        chain:
        
        ```
        Iterables.filter(Throwables.getCausalChain(e), IOException.class));
        ```

        Arguments
        - throwable: the non-null `Throwable` to extract causes from

        Returns
        - an unmodifiable list containing the cause chain starting with `throwable`

        Raises
        - IllegalArgumentException: if there is a loop in the causal chain
        """
        ...


    @staticmethod
    def getCauseAs(throwable: "Throwable", expectedCauseType: type["X"]) -> "X":
        """
        Returns `throwable`'s cause, cast to `expectedCauseType`.
        
        Prefer this method instead of manually casting an exception's cause. For example, `(IOException) e.getCause()` throws a ClassCastException that discards the original
        exception `e` if the cause is not an IOException, but `Throwables.getCauseAs(e, IOException.class)` keeps `e` as the ClassCastException's cause.

        Raises
        - ClassCastException: if the cause cannot be cast to the expected type. The `ClassCastException`'s cause is `throwable`.

        Since
        - 22.0
        """
        ...


    @staticmethod
    def getStackTraceAsString(throwable: "Throwable") -> str:
        """
        Returns a string containing the result of Throwable.toString() toString(), followed by
        the full, recursive stack trace of `throwable`. Note that you probably should not be
        parsing the resulting string; if you need programmatic access to the stack frames, you can call
        Throwable.getStackTrace().
        """
        ...


    @staticmethod
    def lazyStackTrace(throwable: "Throwable") -> list["StackTraceElement"]:
        """
        Returns the stack trace of `throwable`, possibly providing slower iteration over the full
        trace but faster iteration over parts of the trace. Here, "slower" and "faster" are defined in
        comparison to the normal way to access the stack trace, Throwable.getStackTrace()
        throwable.getStackTrace(). Note, however, that this method's special implementation is not
        available for all platforms and configurations. If that implementation is unavailable, this
        method falls back to `getStackTrace`. Callers that require the special implementation can
        check its availability with .lazyStackTraceIsLazy().
        
        The expected (but not guaranteed) performance of the special implementation differs from
        `getStackTrace` in one main way: The `lazyStackTrace` call itself returns quickly
        by delaying the per-stack-frame work until each element is accessed. Roughly speaking:
        
        
          - `getStackTrace` takes `stackSize` time to return but then negligible time to
              retrieve each element of the returned list.
          - `lazyStackTrace` takes negligible time to return but then `1/stackSize` time
              to retrieve each element of the returned list (probably slightly more than `1/stackSize`).
        
        
        Note: The special implementation does not respect calls to Throwable.setStackTrace
        throwable.setStackTrace. Instead, it always reflects the original stack trace from the
        exception's creation.

        Since
        - 19.0

        Deprecated
        - This method is equivalent to Throwable.getStackTrace() on JDK versions past
            JDK 8 and on all Android versions. Use Throwable.getStackTrace() directly, or where
            possible use the `java.lang.StackWalker.walk` method introduced in JDK 9.
        """
        ...


    @staticmethod
    def lazyStackTraceIsLazy() -> bool:
        """
        Returns whether .lazyStackTrace will use the special implementation described in its
        documentation.

        Since
        - 19.0

        Deprecated
        - This method always returns False on JDK versions past JDK 8 and on all Android
            versions.
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.ImmutableRangeMap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.collect import *
from com.google.common.collect.SortedLists import KeyAbsentBehavior
from com.google.common.collect.SortedLists import KeyPresentBehavior
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import DoNotCall
from com.google.errorprone.annotations import DoNotMock
from java.io import InvalidObjectException
from java.io import ObjectInputStream
from java.io import Serializable
from java.util import Collections
from java.util import NoSuchElementException
from java.util.function import BiFunction
from java.util.function import Function
from java.util.stream import Collector
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ImmutableRangeMap(RangeMap, Serializable):
    """
    A RangeMap whose contents will never change, with many other important properties
    detailed at ImmutableCollection.

    Author(s)
    - Louis Wasserman

    Since
    - 14.0
    """

    @staticmethod
    def toImmutableRangeMap(keyFunction: "Function"["T", "Range"["K"]], valueFunction: "Function"["T", "V"]) -> "Collector"["T", Any, "ImmutableRangeMap"["K", "V"]]:
        """
        Returns a `Collector` that accumulates the input elements into a new `ImmutableRangeMap`. As in Builder, overlapping ranges are not permitted.

        Since
        - 23.1
        """
        ...


    @staticmethod
    def of() -> "ImmutableRangeMap"["K", "V"]:
        """
        Returns an empty immutable range map.
        
        **Performance note:** the instance returned is a singleton.
        """
        ...


    @staticmethod
    def of(range: "Range"["K"], value: "V") -> "ImmutableRangeMap"["K", "V"]:
        """
        Returns an immutable range map mapping a single range to a single value.
        """
        ...


    @staticmethod
    def copyOf(rangeMap: "RangeMap"["K", "V"]) -> "ImmutableRangeMap"["K", "V"]:
        ...


    @staticmethod
    def builder() -> "Builder"["K", "V"]:
        """
        Returns a new builder for an immutable range map.
        """
        ...


    def get(self, key: "K") -> "V":
        ...


    def getEntry(self, key: "K") -> "Entry"["Range"["K"], "V"]:
        ...


    def span(self) -> "Range"["K"]:
        ...


    def put(self, range: "Range"["K"], value: "V") -> None:
        """
        Guaranteed to throw an exception and leave the `RangeMap` unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    def putCoalescing(self, range: "Range"["K"], value: "V") -> None:
        """
        Guaranteed to throw an exception and leave the `RangeMap` unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    def putAll(self, rangeMap: "RangeMap"["K", "V"]) -> None:
        """
        Guaranteed to throw an exception and leave the `RangeMap` unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    def clear(self) -> None:
        """
        Guaranteed to throw an exception and leave the `RangeMap` unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    def remove(self, range: "Range"["K"]) -> None:
        """
        Guaranteed to throw an exception and leave the `RangeMap` unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    def merge(self, range: "Range"["K"], value: "V", remappingFunction: "BiFunction"["V", "V", "V"]) -> None:
        """
        Guaranteed to throw an exception and leave the `RangeMap` unmodified.

        Raises
        - UnsupportedOperationException: always

        Since
        - 28.1

        Deprecated
        - Unsupported operation.
        """
        ...


    def asMapOfRanges(self) -> "ImmutableMap"["Range"["K"], "V"]:
        ...


    def asDescendingMapOfRanges(self) -> "ImmutableMap"["Range"["K"], "V"]:
        ...


    def subRangeMap(self, range: "Range"["K"]) -> "ImmutableRangeMap"["K", "V"]:
        ...


    def hashCode(self) -> int:
        ...


    def equals(self, o: "Object") -> bool:
        ...


    def toString(self) -> str:
        ...


    class Builder:
        """
        A builder for immutable range maps. Overlapping ranges are prohibited.

        Since
        - 14.0
        """

        def __init__(self):
            ...


        def put(self, range: "Range"["K"], value: "V") -> "Builder"["K", "V"]:
            """
            Associates the specified range with the specified value.

            Raises
            - IllegalArgumentException: if `range` is empty
            """
            ...


        def putAll(self, rangeMap: "RangeMap"["K", "V"]) -> "Builder"["K", "V"]:
            """
            Copies all associations from the specified range map into this builder.
            """
            ...


        def build(self) -> "ImmutableRangeMap"["K", "V"]:
            """
            Returns an `ImmutableRangeMap` containing the associations previously added to this
            builder.

            Raises
            - IllegalArgumentException: if any two ranges inserted into this builder overlap
            """
            ...



"""
Python module generated from Java source file org.bukkit.inventory.EquipmentSlotGroup

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from java.util import Locale
from java.util.function import Predicate
from org.bukkit.inventory import *
from typing import Any, Callable, Iterable, Tuple


class EquipmentSlotGroup(Predicate):
    """
    Represents a group of EquipmentSlot.
    """

    ANY = get("any", (test) -> true, EquipmentSlot.HAND)
    MAINHAND = get("mainhand", EquipmentSlot.HAND)
    OFFHAND = get("offhand", EquipmentSlot.OFF_HAND)
    HAND = get("hand", (test) -> test == EquipmentSlot.HAND || test == EquipmentSlot.OFF_HAND, EquipmentSlot.HAND)
    FEET = get("feet", EquipmentSlot.FEET)
    LEGS = get("legs", EquipmentSlot.LEGS)
    CHEST = get("chest", EquipmentSlot.CHEST)
    HEAD = get("head", EquipmentSlot.HEAD)
    ARMOR = get("armor", (test) -> test == EquipmentSlot.FEET || test == EquipmentSlot.LEGS || test == EquipmentSlot.CHEST || test == EquipmentSlot.HEAD, EquipmentSlot.CHEST)


    def test(self, test: "EquipmentSlot") -> bool:
        ...


    def toString(self) -> str:
        ...


    def getExample(self) -> "EquipmentSlot":
        """
        Gets an EquipmentSlot which is an example of a slot in this
        group.

        Returns
        - an example slot

        Deprecated
        - for internal compatibility use only
        """
        ...


    @staticmethod
    def getByName(name: str) -> "EquipmentSlotGroup":
        """
        Gets the EquipmentSlotGroup corresponding to the given string.

        Arguments
        - name: group name

        Returns
        - associated group or null
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerItemHeldEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from typing import Any, Callable, Iterable, Tuple


class PlayerItemHeldEvent(PlayerEvent, Cancellable):
    """
    Fired when a player changes their currently held item
    """

    def __init__(self, player: "Player", previous: int, current: int):
        ...


    def getPreviousSlot(self) -> int:
        """
        Gets the previous held slot index

        Returns
        - Previous slot index
        """
        ...


    def getNewSlot(self) -> int:
        """
        Gets the new held slot index

        Returns
        - New slot index
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



from .ServicePriority import ServicePriority
from .EventExecutor import EventExecutor
from .InvalidPluginException import InvalidPluginException
from .SimplePluginManager import SimplePluginManager
from .PluginDescriptionFile import PluginDescriptionFile
from .IllegalPluginAccessException import IllegalPluginAccessException
from .InvalidDescriptionException import InvalidDescriptionException
from .AuthorNagException import AuthorNagException
from .PluginBase import PluginBase
from .PluginAwareness import PluginAwareness
from .TimedRegisteredListener import TimedRegisteredListener
from .RegisteredListener import RegisteredListener
from .PluginLoader import PluginLoader
from .PluginManager import PluginManager
from .PluginLoadOrder import PluginLoadOrder
from .UnknownDependencyException import UnknownDependencyException
from .SimpleServicesManager import SimpleServicesManager
from .RegisteredServiceProvider import RegisteredServiceProvider
from .Plugin import Plugin
from .PluginDescriptionResolver import PluginDescriptionResolver
from .PluginLogger import PluginLogger
from .ServicesManager import ServicesManager


"""
Python module generated from Java source file java.io.DataOutput

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from typing import Any, Callable, Iterable, Tuple


class DataOutput:
    """
    The `DataOutput` interface provides
    for converting data from any of the Java
    primitive types to a series of bytes and
    writing these bytes to a binary stream.
    There is  also a facility for converting
    a `String` into
    <a href="DataInput.html#modified-utf-8">modified UTF-8</a>
    format and writing the resulting series
    of bytes.
    
    For all the methods in this interface that
    write bytes, it is generally True that if
    a byte cannot be written for any reason,
    an `IOException` is thrown.

    Author(s)
    - Frank Yellin

    See
    - java.io.DataOutputStream

    Since
    - 1.0
    """

    def write(self, b: int) -> None:
        """
        Writes to the output stream the eight
        low-order bits of the argument `b`.
        The 24 high-order  bits of `b`
        are ignored.

        Arguments
        - b: the byte to be written.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def write(self, b: list[int]) -> None:
        """
        Writes to the output stream all the bytes in array `b`.
        If `b` is `null`,
        a `NullPointerException` is thrown.
        If `b.length` is zero, then
        no bytes are written. Otherwise, the byte
        `b[0]` is written first, then
        `b[1]`, and so on; the last byte
        written is `b[b.length-1]`.

        Arguments
        - b: the data.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def write(self, b: list[int], off: int, len: int) -> None:
        """
        Writes `len` bytes from array
        `b`, in order,  to
        the output stream.  If `b`
        is `null`, a `NullPointerException`
        is thrown.  If `off` is negative,
        or `len` is negative, or `off+len`
        is greater than the length of the array
        `b`, then an `IndexOutOfBoundsException`
        is thrown.  If `len` is zero,
        then no bytes are written. Otherwise, the
        byte `b[off]` is written first,
        then `b[off+1]`, and so on; the
        last byte written is `b[off+len-1]`.

        Arguments
        - b: the data.
        - off: the start offset in the data.
        - len: the number of bytes to write.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def writeBoolean(self, v: bool) -> None:
        """
        Writes a `boolean` value to this output stream.
        If the argument `v`
        is `True`, the value `(byte)1`
        is written; if `v` is `False`,
        the  value `(byte)0` is written.
        The byte written by this method may
        be read by the `readBoolean`
        method of interface `DataInput`,
        which will then return a `boolean`
        equal to `v`.

        Arguments
        - v: the boolean to be written.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def writeByte(self, v: int) -> None:
        """
        Writes to the output stream the eight low-order
        bits of the argument `v`.
        The 24 high-order bits of `v`
        are ignored. (This means  that `writeByte`
        does exactly the same thing as `write`
        for an integer argument.) The byte written
        by this method may be read by the `readByte`
        method of interface `DataInput`,
        which will then return a `byte`
        equal to `(byte)v`.

        Arguments
        - v: the byte value to be written.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def writeShort(self, v: int) -> None:
        """
        Writes two bytes to the output
        stream to represent the value of the argument.
        The byte values to be written, in the  order
        shown, are:
        ````(byte)(0xff & (v >> 8))
        (byte)(0xff & v)```` 
        The bytes written by this method may be
        read by the `readShort` method
        of interface `DataInput`, which
        will then return a `short` equal
        to `(short)v`.

        Arguments
        - v: the `short` value to be written.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def writeChar(self, v: int) -> None:
        """
        Writes a `char` value, which
        is comprised of two bytes, to the
        output stream.
        The byte values to be written, in the  order
        shown, are:
        ````(byte)(0xff & (v >> 8))
        (byte)(0xff & v)````
        The bytes written by this method may be
        read by the `readChar` method
        of interface `DataInput`, which
        will then return a `char` equal
        to `(char)v`.

        Arguments
        - v: the `char` value to be written.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def writeInt(self, v: int) -> None:
        """
        Writes an `int` value, which is
        comprised of four bytes, to the output stream.
        The byte values to be written, in the  order
        shown, are:
        ````(byte)(0xff & (v >> 24))
        (byte)(0xff & (v >> 16))
        (byte)(0xff & (v >>  8))
        (byte)(0xff & v)````
        The bytes written by this method may be read
        by the `readInt` method of interface
        `DataInput`, which will then
        return an `int` equal to `v`.

        Arguments
        - v: the `int` value to be written.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def writeLong(self, v: int) -> None:
        """
        Writes a `long` value, which is
        comprised of eight bytes, to the output stream.
        The byte values to be written, in the  order
        shown, are:
        ````(byte)(0xff & (v >> 56))
        (byte)(0xff & (v >> 48))
        (byte)(0xff & (v >> 40))
        (byte)(0xff & (v >> 32))
        (byte)(0xff & (v >> 24))
        (byte)(0xff & (v >> 16))
        (byte)(0xff & (v >>  8))
        (byte)(0xff & v)````
        The bytes written by this method may be
        read by the `readLong` method
        of interface `DataInput`, which
        will then return a `long` equal
        to `v`.

        Arguments
        - v: the `long` value to be written.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def writeFloat(self, v: float) -> None:
        """
        Writes a `float` value,
        which is comprised of four bytes, to the output stream.
        It does this as if it first converts this
        `float` value to an `int`
        in exactly the manner of the `Float.floatToIntBits`
        method  and then writes the `int`
        value in exactly the manner of the  `writeInt`
        method.  The bytes written by this method
        may be read by the `readFloat`
        method of interface `DataInput`,
        which will then return a `float`
        equal to `v`.

        Arguments
        - v: the `float` value to be written.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def writeDouble(self, v: float) -> None:
        """
        Writes a `double` value,
        which is comprised of eight bytes, to the output stream.
        It does this as if it first converts this
        `double` value to a `long`
        in exactly the manner of the `Double.doubleToLongBits`
        method  and then writes the `long`
        value in exactly the manner of the  `writeLong`
        method. The bytes written by this method
        may be read by the `readDouble`
        method of interface `DataInput`,
        which will then return a `double`
        equal to `v`.

        Arguments
        - v: the `double` value to be written.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def writeBytes(self, s: str) -> None:
        """
        Writes a string to the output stream.
        For every character in the string
        `s`,  taken in order, one byte
        is written to the output stream.  If
        `s` is `null`, a `NullPointerException`
        is thrown.  If `s.length`
        is zero, then no bytes are written. Otherwise,
        the character `s[0]` is written
        first, then `s[1]`, and so on;
        the last character written is `s[s.length-1]`.
        For each character, one byte is written,
        the low-order byte, in exactly the manner
        of the `writeByte` method . The
        high-order eight bits of each character
        in the string are ignored.

        Arguments
        - s: the string of bytes to be written.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def writeChars(self, s: str) -> None:
        """
        Writes every character in the string `s`,
        to the output stream, in order,
        two bytes per character. If `s`
        is `null`, a `NullPointerException`
        is thrown.  If `s.length`
        is zero, then no characters are written.
        Otherwise, the character `s[0]`
        is written first, then `s[1]`,
        and so on; the last character written is
        `s[s.length-1]`. For each character,
        two bytes are actually written, high-order
        byte first, in exactly the manner of the
        `writeChar` method.

        Arguments
        - s: the string value to be written.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def writeUTF(self, s: str) -> None:
        """
        Writes two bytes of length information
        to the output stream, followed
        by the
        <a href="DataInput.html#modified-utf-8">modified UTF-8</a>
        representation
        of  every character in the string `s`.
        If `s` is `null`,
        a `NullPointerException` is thrown.
        Each character in the string `s`
        is converted to a group of one, two, or
        three bytes, depending on the value of the
        character.
        If a character `c`
        is in the range `&#92;u0001` through
        `&#92;u007f`, it is represented
        by one byte:
        ```(byte)c ```  
        If a character `c` is `&#92;u0000`
        or is in the range `&#92;u0080`
        through `&#92;u07ff`, then it is
        represented by two bytes, to be written
        in the order shown: ````(byte)(0xc0 | (0x1f & (c >> 6)))
        (byte)(0x80 | (0x3f & c))````  If a character
        `c` is in the range `&#92;u0800`
        through `uffff`, then it is
        represented by three bytes, to be written
        in the order shown: ````(byte)(0xe0 | (0x0f & (c >> 12)))
        (byte)(0x80 | (0x3f & (c >>  6)))
        (byte)(0x80 | (0x3f & c))````   First,
        the total number of bytes needed to represent
        all the characters of `s` is
        calculated. If this number is larger than
        `65535`, then a `UTFDataFormatException`
        is thrown. Otherwise, this length is written
        to the output stream in exactly the manner
        of the `writeShort` method;
        after this, the one-, two-, or three-byte
        representation of each character in the
        string `s` is written.  The
        bytes written by this method may be read
        by the `readUTF` method of interface
        `DataInput`, which will then
        return a `String` equal to `s`.

        Arguments
        - s: the string value to be written.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...



"""
Python module generated from Java source file com.google.common.hash.Fingerprint2011

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import VisibleForTesting
from com.google.common.hash import *
from typing import Any, Callable, Iterable, Tuple


class Fingerprint2011(AbstractNonStreamingHashFunction):
    """
    Implementation of Geoff Pike's fingerprint2011 hash function. See Hashing.fingerprint2011
    for information on the behaviour of the algorithm.
    
    On Intel Core2 2.66, on 1000 bytes, fingerprint2011 takes 0.9 microseconds compared to
    fingerprint at 4.0 microseconds and md5 at 4.5 microseconds.
    
    Note to maintainers: This implementation relies on signed arithmetic being bit-wise equivalent
    to unsigned arithmetic in all cases except:
    
    
      - comparisons (signed values can be negative)
      - division (avoided here)
      - shifting (right shift must be unsigned)

    Author(s)
    - gpike@google.com (Geoff Pike)
    """

    def hashBytes(self, input: list[int], off: int, len: int) -> "HashCode":
        ...


    def bits(self) -> int:
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file com.google.common.collect.JdkBackedImmutableMap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.collect import *
from java.util.function import BiConsumer
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class JdkBackedImmutableMap(ImmutableMap):
    """
    Implementation of ImmutableMap backed by a JDK HashMap, which has smartness protecting against
    hash flooding.
    """

    def size(self) -> int:
        ...


    def get(self, key: "Object") -> "V":
        ...


    def forEach(self, action: "BiConsumer"["K", "V"]) -> None:
        ...



"""
Python module generated from Java source file com.google.common.collect.MoreCollectors

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from java.util import NoSuchElementException
from java.util import Optional
from java.util.stream import Collector
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class MoreCollectors:
    """
    Collectors not present in `java.util.stream.Collectors` that are not otherwise associated
    with a `com.google.common` type.

    Author(s)
    - Louis Wasserman

    Since
    - 21.0
    """

    @staticmethod
    def toOptional() -> "Collector"["T", Any, "Optional"["T"]]:
        """
        A collector that converts a stream of zero or one elements to an `Optional`.

        Returns
        - `Optional.of(onlyElement)` if the stream has exactly one element (must not be
            `null`) and returns `Optional.empty()` if it has none.

        Raises
        - IllegalArgumentException: if the stream consists of two or more elements.
        - NullPointerException: if any element in the stream is `null`.
        """
        ...


    @staticmethod
    def onlyElement() -> "Collector"["T", Any, "T"]:
        """
        A collector that takes a stream containing exactly one element and returns that element. The
        returned collector throws an `IllegalArgumentException` if the stream consists of two or
        more elements, and a `NoSuchElementException` if the stream is empty.
        """
        ...



"""
Python module generated from Java source file java.lang.reflect.Type

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.reflect import *
from typing import Any, Callable, Iterable, Tuple


class Type:
    """
    Type is the common superinterface for all types in the Java
    programming language. These include raw types, parameterized types,
    array types, type variables and primitive types.

    Since
    - 1.5

    Unknown Tags
    - 4.1 The Kinds of Types and Values
    - 4.2 Primitive Types and Values
    - 4.3 Reference Types and Values
    - 4.4 Type Variables
    - 4.5 Parameterized Types
    - 4.8 Raw Types
    - 4.9 Intersection Types
    - 10.1 Array Types
    """

    def getTypeName(self) -> str:
        """
        Returns a string describing this type, including information
        about any type parameters.

        Returns
        - a string describing this type

        Since
        - 1.8

        Unknown Tags
        - The default implementation calls `toString`.
        """
        ...



"""
Python module generated from Java source file com.google.common.io.ByteSource

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import Ascii
from com.google.common.base import Optional
from com.google.common.collect import ImmutableList
from com.google.common.hash import Funnels
from com.google.common.hash import HashCode
from com.google.common.hash import HashFunction
from com.google.common.hash import Hasher
from com.google.common.io import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.io import BufferedInputStream
from java.io import ByteArrayInputStream
from java.io import IOException
from java.io import InputStream
from java.io import InputStreamReader
from java.io import OutputStream
from java.io import Reader
from java.nio.charset import Charset
from java.util import Arrays
from java.util import Iterator
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ByteSource:
    """
    A readable source of bytes, such as a file. Unlike an InputStream, a `ByteSource`
    is not an open, stateful stream for input that can be read and closed. Instead, it is an
    immutable *supplier* of `InputStream` instances.
    
    `ByteSource` provides two kinds of methods:
    
    
      - **Methods that return a stream:** These methods should return a *new*, independent
          instance each time they are called. The caller is responsible for ensuring that the
          returned stream is closed.
      - **Convenience methods:** These are implementations of common operations that are
          typically implemented by opening a stream using one of the methods in the first category,
          doing something and finally closing the stream that was opened.
    
    
    **Note:** In general, `ByteSource` is intended to be used for "file-like" sources
    that provide streams that are:
    
    
      - **Finite:** Many operations, such as .size() and .read(), will either
          block indefinitely or fail if the source creates an infinite stream.
      - **Non-destructive:** A *destructive* stream will consume or otherwise alter the
          bytes of the source as they are read from it. A source that provides such streams will not
          be reusable, and operations that read from the stream (including .size(), in some
          implementations) will prevent further operations from completing as expected.

    Author(s)
    - Colin Decker

    Since
    - 14.0
    """

    def asCharSource(self, charset: "Charset") -> "CharSource":
        """
        Returns a CharSource view of this byte source that decodes bytes read from this source
        as characters using the given Charset.
        
        If CharSource.asByteSource is called on the returned source with the same charset,
        the default implementation of this method will ensure that the original `ByteSource` is
        returned, rather than round-trip encoding. Subclasses that override this method should behave
        the same way.
        """
        ...


    def openStream(self) -> "InputStream":
        """
        Opens a new InputStream for reading from this source. This method returns a new,
        independent stream each time it is called.
        
        The caller is responsible for ensuring that the returned stream is closed.

        Raises
        - IOException: if an I/O error occurs while opening the stream
        """
        ...


    def openBufferedStream(self) -> "InputStream":
        """
        Opens a new buffered InputStream for reading from this source. The returned stream is
        not required to be a BufferedInputStream in order to allow implementations to simply
        delegate to .openStream() when the stream returned by that method does not benefit from
        additional buffering (for example, a `ByteArrayInputStream`). This method returns a new,
        independent stream each time it is called.
        
        The caller is responsible for ensuring that the returned stream is closed.

        Raises
        - IOException: if an I/O error occurs while opening the stream

        Since
        - 15.0 (in 14.0 with return type BufferedInputStream)
        """
        ...


    def slice(self, offset: int, length: int) -> "ByteSource":
        """
        Returns a view of a slice of this byte source that is at most `length` bytes long
        starting at the given `offset`. If `offset` is greater than the size of this
        source, the returned source will be empty. If `offset + length` is greater than the size
        of this source, the returned source will contain the slice starting at `offset` and
        ending at the end of this source.

        Raises
        - IllegalArgumentException: if `offset` or `length` is negative
        """
        ...


    def isEmpty(self) -> bool:
        """
        Returns whether the source has zero bytes. The default implementation first checks .sizeIfKnown, returning True if it's known to be zero and False if it's known to be non-zero.
        If the size is not known, it falls back to opening a stream and checking for EOF.
        
        Note that, in cases where `sizeIfKnown` returns zero, it is *possible* that bytes
        are actually available for reading. (For example, some special files may return a size of 0
        despite actually having content when read.) This means that a source may return `True`
        from `isEmpty()` despite having readable content.

        Raises
        - IOException: if an I/O error occurs

        Since
        - 15.0
        """
        ...


    def sizeIfKnown(self) -> "Optional"["Long"]:
        """
        Returns the size of this source in bytes, if the size can be easily determined without actually
        opening the data stream.
        
        The default implementation returns Optional.absent. Some sources, such as a file,
        may return a non-absent value. Note that in such cases, it is *possible* that this method
        will return a different number of bytes than would be returned by reading all of the bytes (for
        example, some special files may return a size of 0 despite actually having content when read).
        
        Additionally, for mutable sources such as files, a subsequent read may return a different
        number of bytes if the contents are changed.

        Since
        - 19.0
        """
        ...


    def size(self) -> int:
        """
        Returns the size of this source in bytes, even if doing so requires opening and traversing an
        entire stream. To avoid a potentially expensive operation, see .sizeIfKnown.
        
        The default implementation calls .sizeIfKnown and returns the value if present. If
        absent, it will fall back to a heavyweight operation that will open a stream, read (or InputStream.skip(long) skip, if possible) to the end of the stream and return the total number
        of bytes that were read.
        
        Note that for some sources that implement .sizeIfKnown to provide a more efficient
        implementation, it is *possible* that this method will return a different number of bytes
        than would be returned by reading all of the bytes (for example, some special files may return
        a size of 0 despite actually having content when read).
        
        In either case, for mutable sources such as files, a subsequent read may return a different
        number of bytes if the contents are changed.

        Raises
        - IOException: if an I/O error occurs while reading the size of this source
        """
        ...


    def copyTo(self, output: "OutputStream") -> int:
        """
        Copies the contents of this byte source to the given `OutputStream`. Does not close
        `output`.

        Returns
        - the number of bytes copied

        Raises
        - IOException: if an I/O error occurs while reading from this source or writing to `output`
        """
        ...


    def copyTo(self, sink: "ByteSink") -> int:
        """
        Copies the contents of this byte source to the given `ByteSink`.

        Returns
        - the number of bytes copied

        Raises
        - IOException: if an I/O error occurs while reading from this source or writing to `sink`
        """
        ...


    def read(self) -> list[int]:
        """
        Reads the full contents of this byte source as a byte array.

        Raises
        - IOException: if an I/O error occurs while reading from this source
        """
        ...


    def read(self, processor: "ByteProcessor"["T"]) -> "T":
        """
        Reads the contents of this byte source using the given `processor` to process bytes as
        they are read. Stops when all bytes have been read or the consumer returns `False`.
        Returns the result produced by the processor.

        Raises
        - IOException: if an I/O error occurs while reading from this source or if `processor` throws an `IOException`

        Since
        - 16.0
        """
        ...


    def hash(self, hashFunction: "HashFunction") -> "HashCode":
        """
        Hashes the contents of this byte source using the given hash function.

        Raises
        - IOException: if an I/O error occurs while reading from this source
        """
        ...


    def contentEquals(self, other: "ByteSource") -> bool:
        """
        Checks that the contents of this byte source are equal to the contents of the given byte
        source.

        Raises
        - IOException: if an I/O error occurs while reading from this source or `other`
        """
        ...


    @staticmethod
    def concat(sources: Iterable["ByteSource"]) -> "ByteSource":
        """
        Concatenates multiple ByteSource instances into a single source. Streams returned from
        the source will contain the concatenated data from the streams of the underlying sources.
        
        Only one underlying stream will be open at a time. Closing the concatenated stream will
        close the open underlying stream.

        Arguments
        - sources: the sources to concatenate

        Returns
        - a `ByteSource` containing the concatenated data

        Since
        - 15.0
        """
        ...


    @staticmethod
    def concat(sources: Iterator["ByteSource"]) -> "ByteSource":
        """
        Concatenates multiple ByteSource instances into a single source. Streams returned from
        the source will contain the concatenated data from the streams of the underlying sources.
        
        Only one underlying stream will be open at a time. Closing the concatenated stream will
        close the open underlying stream.
        
        Note: The input `Iterator` will be copied to an `ImmutableList` when this method
        is called. This will fail if the iterator is infinite and may cause problems if the iterator
        eagerly fetches data for each source when iterated (rather than producing sources that only
        load data through their streams). Prefer using the .concat(Iterable) overload if
        possible.

        Arguments
        - sources: the sources to concatenate

        Returns
        - a `ByteSource` containing the concatenated data

        Raises
        - NullPointerException: if any of `sources` is `null`

        Since
        - 15.0
        """
        ...


    @staticmethod
    def concat(*sources: Tuple["ByteSource", ...]) -> "ByteSource":
        """
        Concatenates multiple ByteSource instances into a single source. Streams returned from
        the source will contain the concatenated data from the streams of the underlying sources.
        
        Only one underlying stream will be open at a time. Closing the concatenated stream will
        close the open underlying stream.

        Arguments
        - sources: the sources to concatenate

        Returns
        - a `ByteSource` containing the concatenated data

        Raises
        - NullPointerException: if any of `sources` is `null`

        Since
        - 15.0
        """
        ...


    @staticmethod
    def wrap(b: list[int]) -> "ByteSource":
        """
        Returns a view of the given byte array as a ByteSource. To view only a specific range
        in the array, use `ByteSource.wrap(b).slice(offset, length)`.
        
        Note that the given byte array may be passed directly to methods on, for example, `OutputStream` (when `copyTo(OutputStream)` is called on the resulting `ByteSource`). This could allow a malicious `OutputStream` implementation to modify the
        contents of the array, but provides better performance in the normal case.

        Since
        - 15.0 (since 14.0 as `ByteStreams.asByteSource(byte[])`).
        """
        ...


    @staticmethod
    def empty() -> "ByteSource":
        """
        Returns an immutable ByteSource that contains no bytes.

        Since
        - 15.0
        """
        ...



"""
Python module generated from Java source file org.joml.Vector4dc

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class Vector4dc:
    """
    Interface to a read-only view of a 4-dimensional vector of double-precision floats.

    Author(s)
    - Kai Burjack
    """

    def x(self) -> float:
        """
        Returns
        - the value of the x component
        """
        ...


    def y(self) -> float:
        """
        Returns
        - the value of the y component
        """
        ...


    def z(self) -> float:
        """
        Returns
        - the value of the z component
        """
        ...


    def w(self) -> float:
        """
        Returns
        - the value of the w component
        """
        ...


    def get(self, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this vector into the supplied ByteBuffer at the current
        buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the vector is stored, use .get(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this vector in `x, y, z, w` order

        Returns
        - the passed in buffer

        See
        - .get(int, ByteBuffer)
        """
        ...


    def get(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this vector into the supplied ByteBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: will receive the values of this vector in `x, y, z, w` order

        Returns
        - the passed in buffer
        """
        ...


    def get(self, buffer: "DoubleBuffer") -> "DoubleBuffer":
        """
        Store this vector into the supplied DoubleBuffer at the current
        buffer DoubleBuffer.position() position.
        
        This method will not increment the position of the given DoubleBuffer.
        
        In order to specify the offset into the DoubleBuffer at which
        the vector is stored, use .get(int, DoubleBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this vector in `x, y, z, w` order

        Returns
        - the passed in buffer

        See
        - .get(int, DoubleBuffer)
        """
        ...


    def get(self, index: int, buffer: "DoubleBuffer") -> "DoubleBuffer":
        """
        Store this vector into the supplied DoubleBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given DoubleBuffer.

        Arguments
        - index: the absolute position into the DoubleBuffer
        - buffer: will receive the values of this vector in `x, y, z, w` order

        Returns
        - the passed in buffer
        """
        ...


    def get(self, buffer: "FloatBuffer") -> "FloatBuffer":
        """
        Store this vector into the supplied FloatBuffer at the current
        buffer FloatBuffer.position() position.
        
        This method will not increment the position of the given FloatBuffer.
        
        In order to specify the offset into the FloatBuffer at which
        the vector is stored, use .get(int, FloatBuffer), taking
        the absolute position as parameter.
        
        Please note that due to this vector storing double values those values will potentially
        lose precision when they are converted to float values before being put into the given FloatBuffer.

        Arguments
        - buffer: will receive the values of this vector in `x, y, z, w` order

        Returns
        - the passed in buffer

        See
        - .get(int, DoubleBuffer)
        """
        ...


    def get(self, index: int, buffer: "FloatBuffer") -> "FloatBuffer":
        """
        Store this vector into the supplied FloatBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given FloatBuffer.
        
        Please note that due to this vector storing double values those values will potentially
        lose precision when they are converted to float values before being put into the given FloatBuffer.

        Arguments
        - index: the absolute position into the FloatBuffer
        - buffer: will receive the values of this vector in `x, y, z, w` order

        Returns
        - the passed in buffer
        """
        ...


    def getf(self, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this vector into the supplied ByteBuffer at the current
        buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the vector is stored, use .get(int, ByteBuffer), taking
        the absolute position as parameter.
        
        Please note that due to this vector storing double values those values will potentially
        lose precision when they are converted to float values before being put into the given ByteBuffer.

        Arguments
        - buffer: will receive the values of this vector in `x, y, z, w` order

        Returns
        - the passed in buffer

        See
        - .get(int, ByteBuffer)
        """
        ...


    def getf(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this vector into the supplied ByteBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.
        
        Please note that due to this vector storing double values those values will potentially
        lose precision when they are converted to float values before being put into the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: will receive the values of this vector in `x, y, z, w` order

        Returns
        - the passed in buffer
        """
        ...


    def getToAddress(self, address: int) -> "Vector4dc":
        """
        Store this vector at the given off-heap memory address.
        
        This method will throw an UnsupportedOperationException when JOML is used with `-Djoml.nounsafe`.
        
        *This method is unsafe as it can result in a crash of the JVM process when the specified address range does not belong to this process.*

        Arguments
        - address: the off-heap address where to store this vector

        Returns
        - this
        """
        ...


    def sub(self, v: "Vector4dc", dest: "Vector4d") -> "Vector4d":
        """
        Subtract the supplied vector from this one and store the result in `dest`.

        Arguments
        - v: the vector to subtract
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def sub(self, v: "Vector4fc", dest: "Vector4d") -> "Vector4d":
        """
        Subtract the supplied vector from this one and store the result in `dest`.

        Arguments
        - v: the vector to subtract
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def sub(self, x: float, y: float, z: float, w: float, dest: "Vector4d") -> "Vector4d":
        """
        Subtract `(x, y, z, w)` from this and store the result in `dest`.

        Arguments
        - x: the x component to subtract
        - y: the y component to subtract
        - z: the z component to subtract
        - w: the w component to subtract
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def add(self, v: "Vector4dc", dest: "Vector4d") -> "Vector4d":
        """
        Add the supplied vector to this one and store the result in `dest`.

        Arguments
        - v: the vector to add
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def add(self, v: "Vector4fc", dest: "Vector4d") -> "Vector4d":
        """
        Add the supplied vector to this one and store the result in `dest`.

        Arguments
        - v: the vector to add
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def add(self, x: float, y: float, z: float, w: float, dest: "Vector4d") -> "Vector4d":
        """
        Add `(x, y, z, w)` to this and store the result in `dest`.

        Arguments
        - x: the x component to subtract
        - y: the y component to subtract
        - z: the z component to subtract
        - w: the w component to subtract
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def fma(self, a: "Vector4dc", b: "Vector4dc", dest: "Vector4d") -> "Vector4d":
        """
        Add the component-wise multiplication of `a * b` to this vector
        and store the result in `dest`.

        Arguments
        - a: the first multiplicand
        - b: the second multiplicand
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def fma(self, a: float, b: "Vector4dc", dest: "Vector4d") -> "Vector4d":
        """
        Add the component-wise multiplication of `a * b` to this vector
        and store the result in `dest`.

        Arguments
        - a: the first multiplicand
        - b: the second multiplicand
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mul(self, v: "Vector4dc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply this vector component-wise by the given Vector4dc and store the result in `dest`.

        Arguments
        - v: the vector to multiply this by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mul(self, v: "Vector4fc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply this vector component-wise by the given Vector4fc and store the result in `dest`.

        Arguments
        - v: the vector to multiply this by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def div(self, v: "Vector4dc", dest: "Vector4d") -> "Vector4d":
        """
        Divide this vector component-wise by the given Vector4dc and store the result in `dest`.

        Arguments
        - v: the vector to divide this by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mul(self, mat: "Matrix4dc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given matrix mat with this vector and store the result in `dest`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply `this` by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mul(self, mat: "Matrix4x3dc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given matrix mat with this vector and store the result in
        `dest`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply the vector with
        - dest: the destination vector to hold the result

        Returns
        - dest
        """
        ...


    def mulGeneric(self, mat: "Matrix4x3dc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given matrix `mat` with this vector and store the result in `dest`.
        
        This method does not make any assumptions or optimizations about the properties of the specified matrix.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply the vector with
        - dest: the destination vector to hold the result

        Returns
        - dest
        """
        ...


    def mulTranslation(self, mat: "Matrix4x3dc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given matrix `mat`, which is assumed to only contain translation,
        with this vector and store the result in `dest`.
        
        This method only works if the given matrix _only_ represents a translation.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply the vector with
        - dest: the destination vector to hold the result

        Returns
        - dest
        """
        ...


    def mul(self, mat: "Matrix4x3fc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given matrix mat with this vector and store the result in
        `dest`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply the vector with
        - dest: the destination vector to hold the result

        Returns
        - dest
        """
        ...


    def mulTranslation(self, mat: "Matrix4x3fc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given matrix `mat`, which is assumed to only contain translation,
        with this vector and store the result in `dest`.
        
        This method only works if the given matrix _only_ represents a translation.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply the vector with
        - dest: the destination vector to hold the result

        Returns
        - dest
        """
        ...


    def mulGeneric(self, mat: "Matrix4x3fc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given matrix `mat` with this vector and store the result in `dest`.
        
        This method does not make any assumptions or optimizations about the properties of the specified matrix.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply the vector with
        - dest: the destination vector to hold the result

        Returns
        - dest
        """
        ...


    def mul(self, mat: "Matrix4fc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given matrix mat with this vector and store the result in `dest`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply `this` by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulGeneric(self, mat: "Matrix4dc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given matrix `mat` with this vector and store the result in `dest`.
        
        This method does not make any assumptions or optimizations about the properties of the specified matrix.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply the vector with
        - dest: the destination vector to hold the result

        Returns
        - dest
        """
        ...


    def mulGeneric(self, mat: "Matrix4fc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given matrix `mat` with this vector and store the result in `dest`.
        
        This method does not make any assumptions or optimizations about the properties of the specified matrix.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply the vector with
        - dest: the destination vector to hold the result

        Returns
        - dest
        """
        ...


    def mulTranspose(self, mat: "Matrix4dc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the transpose of the given matrix `mat` with this vector and store the result in
        `dest`.
        
        Note that this method performs the operation `M^T * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix whose transpose to multiply the vector with
        - dest: the destination vector to hold the result

        Returns
        - dest
        """
        ...


    def mulTranslation(self, mat: "Matrix4dc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given matrix `mat`, which is assumed to only contain translation, with this vector
        and store the result in `dest`.
        
        This method only works if the given matrix _only_ represents a translation.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the affine matrix to multiply the vector with
        - dest: the destination vector to hold the result

        Returns
        - dest
        """
        ...


    def mulTranslation(self, mat: "Matrix4fc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given matrix `mat`, which is assumed to only contain translation, with this vector
        and store the result in `dest`.
        
        This method only works if the given matrix _only_ represents a translation.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the affine matrix to multiply the vector with
        - dest: the destination vector to hold the result

        Returns
        - dest
        """
        ...


    def mulAffine(self, mat: "Matrix4dc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given affine matrix `mat` with this vector and store the result in
        `dest`.
        
        This method only works if the given matrix _only_ represents an affine transformation.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the affine matrix to multiply the vector with
        - dest: the destination vector to hold the result

        Returns
        - dest
        """
        ...


    def mulAffine(self, mat: "Matrix4fc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given affine matrix `mat` with this vector and store the result in
        `dest`.
        
        This method only works if the given matrix _only_ represents an affine transformation.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the affine matrix to multiply the vector with
        - dest: the destination vector to hold the result

        Returns
        - dest
        """
        ...


    def mulAffineTranspose(self, mat: "Matrix4dc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the transpose of the given affine matrix `mat` with this vector and store the result in
        `dest`.
        
        This method only works if the given matrix _only_ represents an affine transformation.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the affine matrix whose transpose to multiply the vector with
        - dest: the destination vector to hold the result

        Returns
        - dest
        """
        ...


    def mulGenericTranspose(self, mat: "Matrix4dc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the transpose of the given matrix `mat` with this vector and store the result in
        `dest`.
        
        This method does not make any assumptions or optimizations about the properties of the specified matrix.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the affine matrix whose transpose to multiply the vector with
        - dest: the destination vector to hold the result

        Returns
        - dest
        """
        ...


    def mulProject(self, mat: "Matrix4dc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given matrix `mat` with this vector, perform perspective division
        and store the result in `dest`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulProject(self, mat: "Matrix4dc", dest: "Vector3d") -> "Vector3d":
        """
        Multiply the given matrix `mat` with this vector, perform perspective division
        and store the `(x, y, z)` result in `dest`.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulProjectGeneric(self, mat: "Matrix4dc", dest: "Vector3d") -> "Vector3d":
        """
        Multiply the given matrix `mat` with this vector, perform perspective division
        and store the `(x, y, z)` result in `dest`.
        
        This method does not make any assumptions or optimizations about the properties of the specified matrix.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulProjectGeneric(self, mat: "Matrix4dc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given matrix `mat` with this vector, perform perspective division
        and store the result in `dest`.
        
        This method does not make any assumptions or optimizations about the properties of the specified matrix.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulProjectTranslation(self, mat: "Matrix4dc", dest: "Vector3d") -> "Vector3d":
        """
        Multiply the given matrix `mat`, which is assumed to only contain translation, with this vector,
        perform perspective division and store the `(x, y, z)` result in `dest`.
        
        This method only works if the given matrix _only_ represents a translation.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulProjectTranslation(self, mat: "Matrix4dc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given matrix `mat`, which is assumed to only contain translation,
        with this vector, perform perspective division and store the result in `dest`.
        
        This method does not make any assumptions or optimizations about the properties of the specified matrix.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulProjectAffine(self, mat: "Matrix4dc", dest: "Vector3d") -> "Vector3d":
        """
        Multiply the given affine matrix `mat`, with this vector,
        perform perspective division and store the `(x, y, z)` result in `dest`.
        
        This method only works if the given matrix _only_ represents an affine transformation.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulProjectAffine(self, mat: "Matrix4dc", dest: "Vector4d") -> "Vector4d":
        """
        Multiply the given affine matrix `mat`, with this vector,
        perform perspective division and store the result in `dest`.
        
        This method only works if the given matrix _only_ represents an affine transformation.
        
        Note that this method performs the operation `M * this`, where `M` is the provided matrix
        and thus interprets `this` as a *column* vector.

        Arguments
        - mat: the matrix to multiply this vector by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulAdd(self, a: "Vector4dc", b: "Vector4dc", dest: "Vector4d") -> "Vector4d":
        """
        Add the component-wise multiplication of `this * a` to `b`
        and store the result in `dest`.

        Arguments
        - a: the multiplicand
        - b: the addend
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulAdd(self, a: float, b: "Vector4dc", dest: "Vector4d") -> "Vector4d":
        """
        Add the component-wise multiplication of `this * a` to `b`
        and store the result in `dest`.

        Arguments
        - a: the multiplicand
        - b: the addend
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mul(self, scalar: float, dest: "Vector4d") -> "Vector4d":
        """
        Multiply this vector by the given scalar value and store the result in `dest`.

        Arguments
        - scalar: the factor to multiply by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def div(self, scalar: float, dest: "Vector4d") -> "Vector4d":
        """
        Divide this vector by the given scalar value and store the result in `dest`.

        Arguments
        - scalar: the factor to divide by
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotate(self, quat: "Quaterniondc", dest: "Vector4d") -> "Vector4d":
        """
        Transform this vector by the given quaternion `quat` and store the result in `dest`.

        Arguments
        - quat: the quaternion to transform this vector
        - dest: will hold the result

        Returns
        - dest

        See
        - Quaterniond.transform(Vector4d)
        """
        ...


    def rotateAxis(self, angle: float, aX: float, aY: float, aZ: float, dest: "Vector4d") -> "Vector4d":
        """
        Rotate this vector the specified radians around the given rotation axis and store the result
        into `dest`.
        
        This vector's `w` component is ignored.
        
        If the rotation axis is either `(1, 0, 0)`, `(0, 1, 0)` or `(0, 0, 1)`,
        then .rotateX(double, Vector4d) rotateX(), .rotateY(double, Vector4d) rotateY() or
        .rotateZ(double, Vector4d) rotateZ(), respectively, should be used instead.

        Arguments
        - angle: the angle in radians
        - aX: the x component of the rotation axis
        - aY: the y component of the rotation axis
        - aZ: the z component of the rotation axis
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateX(self, angle: float, dest: "Vector4d") -> "Vector4d":
        """
        Rotate this vector the specified radians around the X axis and store the result
        into `dest`.

        Arguments
        - angle: the angle in radians
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateY(self, angle: float, dest: "Vector4d") -> "Vector4d":
        """
        Rotate this vector the specified radians around the Y axis and store the result
        into `dest`.

        Arguments
        - angle: the angle in radians
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateZ(self, angle: float, dest: "Vector4d") -> "Vector4d":
        """
        Rotate this vector the specified radians around the Z axis and store the result
        into `dest`.

        Arguments
        - angle: the angle in radians
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def lengthSquared(self) -> float:
        """
        Return the length squared of this vector.

        Returns
        - the length squared
        """
        ...


    def length(self) -> float:
        """
        Return the length of this vector.

        Returns
        - the length
        """
        ...


    def normalize(self, dest: "Vector4d") -> "Vector4d":
        """
        Normalizes this vector and store the result in `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def normalize(self, length: float, dest: "Vector4d") -> "Vector4d":
        """
        Scale this vector to have the given length and store the result in `dest`.

        Arguments
        - length: the desired length
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def normalize3(self, dest: "Vector4d") -> "Vector4d":
        """
        Normalize this vector by computing only the norm of `(x, y, z)` and store the result in `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def distance(self, v: "Vector4dc") -> float:
        """
        Return the distance between this Vector and `v`.

        Arguments
        - v: the other vector

        Returns
        - the distance
        """
        ...


    def distance(self, x: float, y: float, z: float, w: float) -> float:
        """
        Return the distance between `this` vector and `(x, y, z, w)`.

        Arguments
        - x: the x component of the other vector
        - y: the y component of the other vector
        - z: the z component of the other vector
        - w: the w component of the other vector

        Returns
        - the euclidean distance
        """
        ...


    def distanceSquared(self, v: "Vector4dc") -> float:
        """
        Return the square of the distance between this vector and `v`.

        Arguments
        - v: the other vector

        Returns
        - the squared of the distance
        """
        ...


    def distanceSquared(self, x: float, y: float, z: float, w: float) -> float:
        """
        Return the square of the distance between `this` vector and
        `(x, y, z, w)`.

        Arguments
        - x: the x component of the other vector
        - y: the y component of the other vector
        - z: the z component of the other vector
        - w: the w component of the other vector

        Returns
        - the square of the distance
        """
        ...


    def dot(self, v: "Vector4dc") -> float:
        """
        Compute the dot product (inner product) of this vector and `v`.

        Arguments
        - v: the other vector

        Returns
        - the dot product
        """
        ...


    def dot(self, x: float, y: float, z: float, w: float) -> float:
        """
        Compute the dot product (inner product) of this vector and `(x, y, z, w)`.

        Arguments
        - x: the x component of the other vector
        - y: the y component of the other vector
        - z: the z component of the other vector
        - w: the w component of the other vector

        Returns
        - the dot product
        """
        ...


    def angleCos(self, v: "Vector4dc") -> float:
        """
        Return the cosine of the angle between this vector and the supplied vector.
        
        Use this instead of `Math.cos(angle(v))`.

        Arguments
        - v: the other vector

        Returns
        - the cosine of the angle

        See
        - .angle(Vector4dc)
        """
        ...


    def angle(self, v: "Vector4dc") -> float:
        """
        Return the angle between this vector and the supplied vector.

        Arguments
        - v: the other vector

        Returns
        - the angle, in radians

        See
        - .angleCos(Vector4dc)
        """
        ...


    def negate(self, dest: "Vector4d") -> "Vector4d":
        """
        Negate this vector and store the result in `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def min(self, v: "Vector4dc", dest: "Vector4d") -> "Vector4d":
        """
        Set the components of `dest` to be the component-wise minimum of this and the other vector.

        Arguments
        - v: the other vector
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def max(self, v: "Vector4dc", dest: "Vector4d") -> "Vector4d":
        """
        Set the components of `dest` to be the component-wise maximum of this and the other vector.

        Arguments
        - v: the other vector
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def smoothStep(self, v: "Vector4dc", t: float, dest: "Vector4d") -> "Vector4d":
        """
        Compute a smooth-step (i.e. hermite with zero tangents) interpolation
        between `this` vector and the given vector `v` and
        store the result in `dest`.

        Arguments
        - v: the other vector
        - t: the interpolation factor, within `[0..1]`
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def hermite(self, t0: "Vector4dc", v1: "Vector4dc", t1: "Vector4dc", t: float, dest: "Vector4d") -> "Vector4d":
        """
        Compute a hermite interpolation between `this` vector and its
        associated tangent `t0` and the given vector `v`
        with its tangent `t1` and store the result in
        `dest`.

        Arguments
        - t0: the tangent of `this` vector
        - v1: the other vector
        - t1: the tangent of the other vector
        - t: the interpolation factor, within `[0..1]`
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def lerp(self, other: "Vector4dc", t: float, dest: "Vector4d") -> "Vector4d":
        """
        Linearly interpolate `this` and `other` using the given interpolation factor `t`
        and store the result in `dest`.
        
        If `t` is `0.0` then the result is `this`. If the interpolation factor is `1.0`
        then the result is `other`.

        Arguments
        - other: the other vector
        - t: the interpolation factor between 0.0 and 1.0
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def get(self, component: int) -> float:
        """
        Get the value of the specified component of this vector.

        Arguments
        - component: the component, within `[0..3]`

        Returns
        - the value

        Raises
        - IllegalArgumentException: if `component` is not within `[0..3]`
        """
        ...


    def get(self, mode: int, dest: "Vector4i") -> "Vector4i":
        """
        Set the components of the given vector `dest` to those of `this` vector
        using the given RoundingMode.

        Arguments
        - mode: the RoundingMode to use
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def get(self, dest: "Vector4f") -> "Vector4f":
        """
        Set the components of the given vector `dest` to those of `this` vector.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def get(self, dest: "Vector4d") -> "Vector4d":
        """
        Set the components of the given vector `dest` to those of `this` vector.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def maxComponent(self) -> int:
        """
        Determine the component with the biggest absolute value.

        Returns
        - the component index, within `[0..3]`
        """
        ...


    def minComponent(self) -> int:
        """
        Determine the component with the smallest (towards zero) absolute value.

        Returns
        - the component index, within `[0..3]`
        """
        ...


    def floor(self, dest: "Vector4d") -> "Vector4d":
        """
        Compute for each component of this vector the largest (closest to positive
        infinity) `double` value that is less than or equal to that
        component and is equal to a mathematical integer and store the result in
        `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def ceil(self, dest: "Vector4d") -> "Vector4d":
        """
        Compute for each component of this vector the smallest (closest to negative
        infinity) `double` value that is greater than or equal to that
        component and is equal to a mathematical integer and store the result in
        `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def round(self, dest: "Vector4d") -> "Vector4d":
        """
        Compute for each component of this vector the closest double that is equal to
        a mathematical integer, with ties rounding to positive infinity and store
        the result in `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def isFinite(self) -> bool:
        """
        Determine whether all components are finite floating-point values, that
        is, they are not Double.isNaN() NaN and not
        Double.isInfinite() infinity.

        Returns
        - `True` if all components are finite floating-point values;
                `False` otherwise
        """
        ...


    def absolute(self, dest: "Vector4d") -> "Vector4d":
        """
        Compute the absolute of each of this vector's components
        and store the result into `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def equals(self, v: "Vector4dc", delta: float) -> bool:
        """
        Compare the vector components of `this` vector with the given vector using the given `delta`
        and return whether all of them are equal within a maximum difference of `delta`.
        
        Please note that this method is not used by any data structure such as ArrayList HashSet or HashMap
        and their operations, such as ArrayList.contains(Object) or HashSet.remove(Object), since those
        data structures only use the Object.equals(Object) and Object.hashCode() methods.

        Arguments
        - v: the other vector
        - delta: the allowed maximum difference

        Returns
        - `True` whether all of the vector components are equal; `False` otherwise
        """
        ...


    def equals(self, x: float, y: float, z: float, w: float) -> bool:
        """
        Compare the vector components of `this` vector with the given `(x, y, z, w)`
        and return whether all of them are equal.

        Arguments
        - x: the x component to compare to
        - y: the y component to compare to
        - z: the z component to compare to
        - w: the w component to compare to

        Returns
        - `True` if all the vector components are equal
        """
        ...



"""
Python module generated from Java source file java.nio.file.attribute.UserPrincipal

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.nio.file.attribute import *
from java.security import Principal
from typing import Any, Callable, Iterable, Tuple


class UserPrincipal(Principal):


